package edu.uoc.jsev.b4b.ui;

import android.support.test.espresso.intent.rule.IntentsTestRule;
import android.support.test.espresso.matcher.ViewMatchers;
import android.support.test.filters.LargeTest;
import android.support.test.runner.AndroidJUnit4;

import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.ui.login.LoginActivity;
import edu.uoc.jsev.b4b.ui.main.MainActivity;
import edu.uoc.jsev.b4b.ui.registration.RegistrationActivity;

import static android.support.test.espresso.Espresso.onView;
import static android.support.test.espresso.action.ViewActions.click;
import static android.support.test.espresso.action.ViewActions.closeSoftKeyboard;
import static android.support.test.espresso.action.ViewActions.typeText;
import static android.support.test.espresso.assertion.ViewAssertions.matches;
import static android.support.test.espresso.intent.Intents.intended;
import static android.support.test.espresso.intent.matcher.IntentMatchers.hasComponent;
import static android.support.test.espresso.matcher.RootMatchers.withDecorView;
import static android.support.test.espresso.matcher.ViewMatchers.isDescendantOfA;
import static android.support.test.espresso.matcher.ViewMatchers.isDisplayed;
import static android.support.test.espresso.matcher.ViewMatchers.withId;
import static android.support.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.Matchers.allOf;
import static org.hamcrest.Matchers.not;

/**
 * Clase de prueba para la actividad de login.
 */
@RunWith(AndroidJUnit4.class)
@LargeTest
public class LoginActivityTest extends BaseTest {

    // Actividad inicial de la prueba.
    @Rule
    public IntentsTestRule<LoginActivity> mActivityTestRule = new IntentsTestRule<>(LoginActivity.class);

    /**
     * Comprueba que se muestran los mensajes correspondientes
     * si se intenta acceder sin rellenar los campos obligatorios.
     */
    @Test
    public void emptyLoginTest() {
        // Se pulsa sobre el botón "Acceder".
        onView(ViewMatchers.withId(R.id.sign_in_button))
                .perform(closeSoftKeyboard(), click());

        // Validaciones
        onView(allOf(isDescendantOfA(withId(R.id.email)), withText(R.string.error_mandatory_field)))
                .check(matches(isDisplayed()));
        onView(allOf(isDescendantOfA(withId(R.id.password)), withText(R.string.error_mandatory_field)))
                .check(matches(isDisplayed()));
    }

    /**
     * Comprueba que se muestran los mensajes correspondientes
     * si se intenta acceder habiendo rellenado el campo email
     * con un formato incorrecto.
     */
    @Test
    public void emailFormatLoginTest() {
        // Se rellena el email con un formato no válido.
        onView(withId(R.id.email_text))
                .perform(typeText(USER_EMAIL.replaceAll("@", "")), closeSoftKeyboard());
        // Se rellena el password.
        onView(withId(R.id.password_text))
                .perform(typeText(USER_PASSWD), closeSoftKeyboard());
        // Se pulsa sobre "Acceder".
        onView(withId(R.id.sign_in_button))
                .perform(closeSoftKeyboard(), click());

        // Validaciones.
        onView(allOf(isDescendantOfA(withId(R.id.email)), withText(R.string.error_invalid_email)))
                .check(matches(isDisplayed()));
    }

    /**
     * Comprueba que se muestran los mensajes correspondientes
     * si se intenta acceder con un usuario/contraseña incorrectos.
     */
    @Test
    public void invalidLoginTest() {
        // Se rellena el email.
        onView(withId(R.id.email_text))
                .perform(typeText(USER_EMAIL), closeSoftKeyboard());
        // Se rellena el password.
        onView(withId(R.id.password_text))
                .perform(typeText(USER_PASSWD.substring(1)), closeSoftKeyboard());
        // Se pulsa sobre "Acceder".
        onView(withId(R.id.sign_in_button))
                .perform(closeSoftKeyboard(), click());

        // Validaciones.
        onView(withText(R.string.error_incorrect_login))
                .inRoot(withDecorView(not(mActivityTestRule.getActivity().getWindow().getDecorView())))
                .check(matches(isDisplayed()));
    }

    /**
     * Comprueba que se navega correctamente a la pantalla de registro.
     */
    @Test
    public void navigateToRegistrationTest() {
        // Pulsamos sobre la etiqueta para ir a la pantalla de registro.
        onView(withId(R.id.not_registered_label))
                .perform(closeSoftKeyboard(), click());

        // Validaciones.
        intended(hasComponent(RegistrationActivity.class.getName()));
    }

    /**
     * Comprueba que se accede a la aplicación si se introducen
     * unas credenciales válidas.
     */
    @Test
    public void validLoginTest() {
        // Se rellena el email.
        onView(withId(R.id.email_text))
                .perform(typeText(USER_EMAIL), closeSoftKeyboard());
        // Se rellena el password.
        onView(withId(R.id.password_text))
                .perform(typeText(USER_PASSWD), closeSoftKeyboard());
        // Se pulsa sobre "Acceder".
        onView(withId(R.id.sign_in_button))
                .perform(click());

        // Validamos que se ha accedido a la pantalla principal
        // y que el usuario está autenticado.
        intended(hasComponent(MainActivity.class.getName()));
        assertLoggedIn(USER_EMAIL);
    }
}
