package edu.uoc.jsev.b4b.ui;

import android.support.test.espresso.contrib.DrawerActions;
import android.support.test.espresso.contrib.DrawerMatchers;
import android.support.test.espresso.contrib.NavigationViewActions;
import android.support.test.espresso.intent.rule.IntentsTestRule;
import android.support.test.espresso.matcher.ViewMatchers;
import android.support.test.filters.LargeTest;
import android.support.test.runner.AndroidJUnit4;
import android.view.Gravity;

import com.backendless.Backendless;

import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.ui.login.LoginActivity;
import edu.uoc.jsev.b4b.ui.main.MainActivity;

import static android.support.test.espresso.Espresso.onView;
import static android.support.test.espresso.action.ViewActions.click;
import static android.support.test.espresso.assertion.ViewAssertions.matches;
import static android.support.test.espresso.intent.Intents.intended;
import static android.support.test.espresso.intent.matcher.IntentMatchers.hasComponent;
import static android.support.test.espresso.matcher.RootMatchers.isDialog;
import static android.support.test.espresso.matcher.ViewMatchers.isDisplayed;
import static android.support.test.espresso.matcher.ViewMatchers.withId;
import static android.support.test.espresso.matcher.ViewMatchers.withText;

/**
 * Clase de prueba para verificar la desconexión de la aplicación.
 * La prueba asume que los servicios de localización están desactivados,
 * para poder cerrar el diálogo que aparece al entrar en la pantalla principal
 * de la aplicación preguntando si se desean activar.
 */
@RunWith(AndroidJUnit4.class)
@LargeTest
public class LogoutTest extends BaseTest {

    // Actividad inicial de la prueba. Antes de lanzar la prueba autenticamos al usuario.
    @Rule
    public IntentsTestRule<MainActivity> mActivityTestRule = new IntentsTestRule<MainActivity>(MainActivity.class) {
        @Override
        protected void beforeActivityLaunched() {
            super.beforeActivityLaunched();
            Backendless.UserService.login(USER_EMAIL, USER_PASSWD);
        }
    };

    /**
     * Comprueba que el cierre de sesión es correcto.
     */
    @Test
    public void logoutTest() {
        // Cerramos el diálogo de activación de los servicios de localización.
        onView(withText(android.R.string.cancel))
                .inRoot(isDialog())
                .check(matches(isDisplayed()))
                .perform(click());

        // Verificamos que estamos en la pantalla principal, comprobando
        // el estado del menú de navegación lateral, y lo abrimos.
        onView(ViewMatchers.withId(R.id.drawer_layout))
                .check(matches(DrawerMatchers.isClosed(Gravity.LEFT)))
                .perform(DrawerActions.open())
                .check(matches(DrawerMatchers.isOpen()));

        // Pulsamos sobre la opción de salir.
        onView(withId(R.id.nav_view))
                .perform(NavigationViewActions.navigateTo(R.id.nav_exit));

        // Confirmamos que queremos salir.
        onView(withText(android.R.string.ok))
                .inRoot(isDialog())
                .check(matches(isDisplayed()))
                .perform(click());

        // Comprobamos que navegamos a la pantalla de login
        // y que el usuario ya no está autenticado.
        intended(hasComponent(LoginActivity.class.getName()));
        assertLoggedOut();
    }
}
