package edu.uoc.jsev.b4b.ui;


import android.graphics.Rect;
import android.support.test.espresso.UiController;
import android.support.test.espresso.ViewAction;
import android.support.test.espresso.contrib.DrawerMatchers;
import android.support.test.espresso.contrib.RecyclerViewActions;
import android.support.test.espresso.intent.rule.IntentsTestRule;
import android.support.test.espresso.matcher.BoundedMatcher;
import android.support.test.espresso.matcher.ViewMatchers;
import android.support.test.runner.AndroidJUnit4;
import android.support.test.uiautomator.UiObject;
import android.support.test.uiautomator.UiObjectNotFoundException;
import android.support.test.uiautomator.UiSelector;
import android.test.suitebuilder.annotation.LargeTest;
import android.view.Gravity;
import android.view.View;
import android.widget.SeekBar;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.ui.login.LoginActivity;
import edu.uoc.jsev.b4b.ui.main.MainActivity;
import edu.uoc.jsev.b4b.ui.map.userdetail.UserDetailActivity;
import edu.uoc.jsev.b4b.ui.mybooks.bookdetail.BookDetailActivity;

import static android.support.test.espresso.Espresso.onView;
import static android.support.test.espresso.action.ViewActions.click;
import static android.support.test.espresso.action.ViewActions.closeSoftKeyboard;
import static android.support.test.espresso.action.ViewActions.pressImeActionButton;
import static android.support.test.espresso.action.ViewActions.typeText;
import static android.support.test.espresso.assertion.ViewAssertions.matches;
import static android.support.test.espresso.intent.Intents.intended;
import static android.support.test.espresso.intent.matcher.IntentMatchers.hasComponent;
import static android.support.test.espresso.intent.matcher.IntentMatchers.hasExtraWithKey;
import static android.support.test.espresso.matcher.RootMatchers.isDialog;
import static android.support.test.espresso.matcher.ViewMatchers.hasDescendant;
import static android.support.test.espresso.matcher.ViewMatchers.isDisplayed;
import static android.support.test.espresso.matcher.ViewMatchers.withEffectiveVisibility;
import static android.support.test.espresso.matcher.ViewMatchers.withId;
import static android.support.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.Matchers.allOf;
import static org.hamcrest.Matchers.containsString;

/**
 * Clase de prueba para verificar el proceso
 * de búsqueda de usuarios sobre el mapa.
 * La prueba asume que los servicios de localización están desactivados,
 * para poder cerrar el diálogo que aparece al entrar en la pantalla principal
 * de la aplicación preguntando si se desean activar.
 */
@LargeTest
@RunWith(AndroidJUnit4.class)
public class SearchTest extends BaseTest {

    // Constantes para rellenar los campos de la pantalla.
    private static final String SEARCH_BOOK_TITLE = "DragonLance";
    private static final String SEARCH_BOOK_AUTHOR = "Weis";
    private static final int SEARCH_DISTANCE = 930;
    private static final String SEARCH_USER_NAME = "Javier Sevillano";

    // Actividad inicial de la prueba.
    @Rule
    public IntentsTestRule<LoginActivity> mActivityTestRule = new IntentsTestRule<>(LoginActivity.class);

    /**
     * Prueba principal. Realiza lo siguiente:
     *      - Autentica al usuario.
     *      - Abre el filtro de búsqueda.
     *      - Realiza una búsqueda.
     *      - Pulsa sobre un marcador del mapa.
     *      - Navega al detalle de un usuario.
     *      - Navega al detalla de un libro del usuario.
     */
    @Test
    public void searchTest() throws UiObjectNotFoundException {
        login();
        navigateToSearch();
        search();
        navigateToUserDetail();
        navigateToBookDetail();
        assertBookDetail();
    }

    private void login() {
        // Autenticamos al usuario.
        onView(ViewMatchers.withId(R.id.email_text))
                .perform(typeText(USER_EMAIL), closeSoftKeyboard());
        onView(withId(R.id.password_text))
                .perform(typeText(USER_PASSWD), pressImeActionButton());
    }

    private void navigateToSearch() {
        // Verificamos que se ha navegado a la pantalla principal
        // y que el usuario está autenticado.
        intended(hasComponent(MainActivity.class.getName()));
        registerIdlingResource();
        assertLoggedIn(USER_EMAIL);

        // Verificamos que se está mostrando el mapa y abrimos el filtro de búsqueda.
        onView(withText(android.R.string.cancel))
                .inRoot(isDialog())
                .check(matches(isDisplayed()))
                .perform(click());

        onView(withId(R.id.drawer_layout))
                .check(matches(DrawerMatchers.isClosed(Gravity.LEFT)));

        onView(withId(R.id.map))
                .check(matches(isDisplayed()));

        onView(withId(R.id.fab))
                .check(matches(isDisplayed()))
                .perform(click());
    }

    private void search() {
        // Verificamos que el filtro de búsqueda está abierto.
        onView(withId(R.id.bottom_sheet_title))
                .check(matches(allOf(withEffectiveVisibility(ViewMatchers.Visibility.VISIBLE), withText(R.string.title_fragment_search))));

        // Introducimos los criterios de búsqueda y buscamos.
        onView(withId(R.id.title_text))
                .perform(typeText(SEARCH_BOOK_TITLE), closeSoftKeyboard());
        onView(withId(R.id.author_text))
                .perform(typeText(SEARCH_BOOK_AUTHOR), closeSoftKeyboard());
        onView(withId(R.id.distance_seekbar))
                .perform(setProgress(SEARCH_DISTANCE))
                .check(matches(withProgress(SEARCH_DISTANCE)));

        onView(withId(R.id.search_button))
                .perform(click());
    }

    private void navigateToUserDetail() throws UiObjectNotFoundException {
        // La interacción con el mapa se debe simular con UI Automator,
        // ya que con Espresso no funciona correctamente.

        // Pulsamos sobre un marcador, para que se muestre información
        // adicional del usuario.
        UiObject marker = mUiDevice.findObject(new UiSelector().descriptionContains(SEARCH_USER_NAME));
        marker.click();

        // Pulsamos sobre el globo de información, para navegar al detalle del usuario.
        Rect markerRect = marker.getBounds();
        mUiDevice.click(markerRect.centerX(), markerRect.top - 30);

        sleep(1000);
    }

    private void navigateToBookDetail() {
        // Verificamos que nos encontramos en la pantalla de detalle de usuario.
        intended(allOf(hasComponent(UserDetailActivity.class.getName()), hasExtraWithKey(UserDetailActivity.EXTRA_USER_DATA)));
        registerIdlingResource();

        // Pulsamos sobre un libro del usuario, para navegar a su detalle.
        onView(withId(R.id.list))
                .check(matches(isDisplayed()))
                .perform(RecyclerViewActions.actionOnItem(hasDescendant(withText(containsString(SEARCH_BOOK_TITLE))), click()));
    }

    private void assertBookDetail() {
        // Verificamos que nos encontramos en la pantalla de detalle del libro.
        intended(allOf(hasComponent(BookDetailActivity.class.getName()), hasExtraWithKey(BookDetailActivity.EXTRA_BOOK_DATA)));
        onView(withId(R.id.author))
                .check(matches(withText(containsString(SEARCH_BOOK_AUTHOR))));
    }

    // Action y matcher personalizados para la barra de progreso
    // que se utiliza en el filtro para introducir la distancia
    // del radio de búsqueda.
    public static ViewAction setProgress(final int progress) {
        return new ViewAction() {
            @Override
            public void perform(UiController uiController, View view) {
                SeekBar seekBar = (SeekBar) view;
                seekBar.setProgress(progress);
            }
            @Override
            public String getDescription() {
                return "Set a progress on a SeekBar";
            }
            @Override
            public Matcher<View> getConstraints() {
                return ViewMatchers.isAssignableFrom(SeekBar.class);
            }
        };
    }

    public static Matcher<View> withProgress(final int expectedProgress) {
        return new BoundedMatcher<View, SeekBar>(SeekBar.class) {
            @Override
            public void describeTo(Description description) {
                description.appendText("expected: ");
                description.appendText(String.valueOf(expectedProgress));
            }

            @Override
            public boolean matchesSafely(SeekBar seekBar) {
                return seekBar.getProgress() == expectedProgress;
            }
        };
    }
}
