package edu.uoc.jsev.b4b.util;

import android.app.Activity;
import android.os.Handler;
import android.support.annotation.IdRes;
import android.support.annotation.NonNull;
import android.support.test.espresso.IdlingResource;
import android.view.View;

import java.lang.ref.WeakReference;

/**
 * Clase para indicar a Espresso cuándo la aplicación
 * se encuentra realizando una operación en segundo plano.
 * El criterio utilizado es que se indicará que la aplicación
 * está ocupada mientras una determinada vista esté activa
 * (un indicador de progreso, por ejemplo), y dejará de estar
 * activa cuando dicha vista se oculte.
 */
public class ViewVisibilityIdlingResource implements IdlingResource {

    private static final int IDLE_POLL_DELAY_MILLIS = 1500;

    private final String mName;
    private final WeakReference<View> mView;

    private ResourceCallback mResourceCallback;

    public ViewVisibilityIdlingResource(@NonNull Activity activity, @IdRes int viewId) {
        this.mView = new WeakReference<>(activity.findViewById(viewId));
        this.mName = ViewVisibilityIdlingResource.class.getName() + "_" + System.currentTimeMillis();
    }

    @Override
    public String getName() {
        return mName;
    }

    @Override
    public boolean isIdleNow() {
        boolean idle = mView.get().getVisibility() != View.VISIBLE;
        if(idle && mResourceCallback != null) {
            mResourceCallback.onTransitionToIdle();
        }
        else {
            // Forzamos a chequear el estado de la aplicación cada cierto intervalo de tiempo.
            // Por defecto, si isIdleNow() devuelve false, Espresso pregunta por el estado de
            // la aplicación con una frecuencia demasiado baja (varios segundos), lo que ralentiza
            // los tests y no devuelve resultados fiables. Por ello, reducimos, manualmente,
            // el intervalo de chequeo.
            new Handler().postDelayed(new Runnable() {
                @Override
                public void run() {
                    isIdleNow();
                }
            }, IDLE_POLL_DELAY_MILLIS);
        }

        return idle;
    }

    @Override
    public void registerIdleTransitionCallback(ResourceCallback callback) {
        this.mResourceCallback = callback;
    }
}
