package edu.uoc.jsev.b4b;

import edu.uoc.jsev.b4b.data.backendless.service.BookService;
import edu.uoc.jsev.b4b.data.backendless.service.CommonsService;
import edu.uoc.jsev.b4b.data.backendless.service.SearchService;
import edu.uoc.jsev.b4b.data.backendless.service.SwapService;
import edu.uoc.jsev.b4b.data.backendless.service.UserService;
import edu.uoc.jsev.b4b.ui.login.LoginContract;
import edu.uoc.jsev.b4b.ui.login.LoginInteractor;
import edu.uoc.jsev.b4b.ui.login.LoginInteractorImpl;
import edu.uoc.jsev.b4b.ui.login.LoginPresenter;
import edu.uoc.jsev.b4b.ui.main.MainContract;
import edu.uoc.jsev.b4b.ui.main.MainInteractor;
import edu.uoc.jsev.b4b.ui.main.MainInteractorImpl;
import edu.uoc.jsev.b4b.ui.main.MainPresenter;
import edu.uoc.jsev.b4b.ui.map.MapContract;
import edu.uoc.jsev.b4b.ui.map.MapInteractor;
import edu.uoc.jsev.b4b.ui.map.MapInteractorImpl;
import edu.uoc.jsev.b4b.ui.map.MapPresenter;
import edu.uoc.jsev.b4b.ui.map.search.SearchContract;
import edu.uoc.jsev.b4b.ui.map.search.SearchInteractor;
import edu.uoc.jsev.b4b.ui.map.search.SearchInteractorImpl;
import edu.uoc.jsev.b4b.ui.map.search.SearchPresenter;
import edu.uoc.jsev.b4b.ui.map.userdetail.UserDetailContract;
import edu.uoc.jsev.b4b.ui.map.userdetail.UserDetailInteractor;
import edu.uoc.jsev.b4b.ui.map.userdetail.UserDetailInteractorImpl;
import edu.uoc.jsev.b4b.ui.map.userdetail.UserDetailPresenter;
import edu.uoc.jsev.b4b.ui.mybooks.MyBooksContract;
import edu.uoc.jsev.b4b.ui.mybooks.MyBooksInteractor;
import edu.uoc.jsev.b4b.ui.mybooks.MyBooksInteractorImpl;
import edu.uoc.jsev.b4b.ui.mybooks.MyBooksPresenter;
import edu.uoc.jsev.b4b.ui.mybooks.bookdetail.BookDetailContract;
import edu.uoc.jsev.b4b.ui.mybooks.bookdetail.BookDetailInteractor;
import edu.uoc.jsev.b4b.ui.mybooks.bookdetail.BookDetailInteractorImpl;
import edu.uoc.jsev.b4b.ui.mybooks.bookdetail.BookDetailPresenter;
import edu.uoc.jsev.b4b.ui.mybooks.newbook.NewBookContract;
import edu.uoc.jsev.b4b.ui.mybooks.newbook.NewBookInteractor;
import edu.uoc.jsev.b4b.ui.mybooks.newbook.NewBookInteractorImpl;
import edu.uoc.jsev.b4b.ui.mybooks.newbook.NewBookPresenter;
import edu.uoc.jsev.b4b.ui.registration.RegistrationContract;
import edu.uoc.jsev.b4b.ui.registration.RegistrationInteractor;
import edu.uoc.jsev.b4b.ui.registration.RegistrationInteractorImpl;
import edu.uoc.jsev.b4b.ui.registration.RegistrationPresenter;
import edu.uoc.jsev.b4b.ui.splash.SplashContract;
import edu.uoc.jsev.b4b.ui.splash.SplashInteractor;
import edu.uoc.jsev.b4b.ui.splash.SplashInteractorImpl;
import edu.uoc.jsev.b4b.ui.splash.SplashPresenter;
import edu.uoc.jsev.b4b.ui.swaps.SwapsContract;
import edu.uoc.jsev.b4b.ui.swaps.SwapsInteractor;
import edu.uoc.jsev.b4b.ui.swaps.SwapsInteractorImpl;
import edu.uoc.jsev.b4b.ui.swaps.SwapsPresenter;
import edu.uoc.jsev.b4b.ui.swaps.swapdetail.SwapDetailContract;
import edu.uoc.jsev.b4b.ui.swaps.swapdetail.SwapDetailInteractor;
import edu.uoc.jsev.b4b.ui.swaps.swapdetail.SwapDetailInteractorImpl;
import edu.uoc.jsev.b4b.ui.swaps.swapdetail.SwapDetailPresenter;

/**
 * Clase auxiliar para inyectar las dependencias en las clases.
 * El proyecto no utiliza Dagger, por lo que esta clase es una
 * manera de simular, en cierto modo, la DI.
 */
public class Injection {

    /**
     * Proporciona la implementación de un presentador para la pantalla de splash.
     */
    public static SplashContract.Presenter provideSplashPresenter(SplashContract.View view) {
        return new SplashPresenter(view, provideSplashInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla de login.
     */
    public static LoginContract.Presenter provideLoginPresenter(LoginContract.View view) {
        return new LoginPresenter(view, provideLoginInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla de registro de usuarios.
     */
    public static RegistrationContract.Presenter provideRegistrationPresenter(RegistrationContract.View view) {
        return new RegistrationPresenter(view, provideRegistrationInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla principal.
     */
    public static MainContract.Presenter provideMainPresenter(MainContract.View view) {
        return new MainPresenter(view, provideMainInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla del mapa.
     */
    public static MapContract.Presenter provideMapPresenter(MapContract.View view) {
        return new MapPresenter(view, provideMapInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la búsqueda de usuarios/libros.
     */
    public static SearchContract.Presenter provideSearchPresenter(SearchContract.View view) {
        return new SearchPresenter(view, provideSearchInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla de detalle de usuario.
     */
    public static UserDetailContract.Presenter provideUserDetailPresenter(UserDetailContract.View view) {
        return new UserDetailPresenter(view, provideUserDetailInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla con el listado de libros del usuario.
     */
    public static MyBooksContract.Presenter provideMyBooksPresenter(MyBooksContract.View view) {
        return new MyBooksPresenter(view, provideMyBooksInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla de nuevo libro.
     */
    public static NewBookContract.Presenter provideNewBookPresenter(NewBookContract.View view) {
        return new NewBookPresenter(view, provideNewBookInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla de detalle de libro.
     */
    public static BookDetailContract.Presenter provideBookDetailPresenter(BookDetailContract.View view) {
        return new BookDetailPresenter(view, provideBookDetailInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla con el listado de intercambios.
     */
    public static SwapsContract.Presenter provideSwapsPresenter(SwapsContract.View view) {
        return new SwapsPresenter(view, provideSwapsInteractor());
    }

    /**
     * Proporciona la implementación de un presentador para la pantalla de detalle de intercambio.
     */
    public static SwapDetailContract.Presenter provideSwapDetailPresenter(SwapDetailContract.View view) {
        return new SwapDetailPresenter(view, provideSwapDetailInteractor());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla de splash.
     */
    public static SplashInteractor provideSplashInteractor() {
        return SplashInteractorImpl.getInstance(provideUserService(), provideCommonsService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla de login.
     */
    public static LoginInteractor provideLoginInteractor() {
        return LoginInteractorImpl.getInstance(provideUserService(), provideCommonsService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla de registro de usuarios.
     */
    public static RegistrationInteractor provideRegistrationInteractor() {
        return RegistrationInteractorImpl.getInstance(provideUserService(), provideCommonsService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla principal.
     */
    public static MainInteractor provideMainInteractor() {
        return MainInteractorImpl.getInstance(provideUserService(), provideCommonsService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla con el listado de libros del usuario.
     */
    public static MyBooksInteractor provideMyBooksInteractor() {
        return MyBooksInteractorImpl.getInstance(provideUserService(), provideCommonsService(), provideBookService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla de nuevo libro.
     */
    public static NewBookInteractor provideNewBookInteractor() {
        return NewBookInteractorImpl.getInstance(provideUserService(), provideCommonsService(), provideBookService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla de detalle de libro.
     */
    public static BookDetailInteractor provideBookDetailInteractor() {
        return BookDetailInteractorImpl.getInstance(provideUserService(),
                provideCommonsService(), provideBookService(), provideSwapService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla del mapa.
     */
    public static MapInteractor provideMapInteractor() {
        return MapInteractorImpl.getInstance(provideSearchService(), provideCommonsService());
    }

    /**
     * Proporciona la implementación de un interactor para la búsqueda de usuarios/libros.
     */
    public static SearchInteractor provideSearchInteractor() {
        return SearchInteractorImpl.getInstance(provideUserService(), provideCommonsService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla de detalle de usuario.
     */
    public static UserDetailInteractor provideUserDetailInteractor() {
        return UserDetailInteractorImpl.getInstance(provideUserService(), provideCommonsService(), provideBookService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla con el listado de intercambios.
     */
    public static SwapsInteractor provideSwapsInteractor() {
        return SwapsInteractorImpl.getInstance(provideUserService(), provideCommonsService(), provideSwapService());
    }

    /**
     * Proporciona la implementación de un interactor para la pantalla de detalle de intercambio.
     */
    public static SwapDetailInteractor provideSwapDetailInteractor() {
        return SwapDetailInteractorImpl.getInstance(provideUserService(), provideCommonsService(), provideSwapService());
    }

    /**
     * Proporciona la implementación de un servicio de acceso al repositorio de libros.
     */
    public static BookService provideBookService() {
        return BookService.getInstance();
    }

    /**
     * Proporciona la implementación servicios horizontales, tales como el envío
     * de correo electrónico, la carga de imágenes en el servidor o el acceso al
     * repositorio de géneros de libros e idiomas.
     */
    public static CommonsService provideCommonsService() {
        return CommonsService.getInstance();
    }

    /**
     * Proporciona la implementación de un servicio de búsqueda de libros.
     */
    public static SearchService provideSearchService() {
        return SearchService.getInstance();
    }

    /**
     * Proporciona la implementación de un servicio de acceso al repositorio de intercambios.
     */
    public static SwapService provideSwapService() {
        return SwapService.getInstance();
    }

    /**
     * Proporciona la implementación de un servicio de acceso al repositorio de usuarios.
     */
    public static UserService provideUserService() {
        return UserService.getInstance();
    }
}
