package edu.uoc.jsev.b4b.data.openlibrary.service;

import java.util.Map;

import edu.uoc.jsev.b4b.data.openlibrary.model.OpenLibraryBook;
import retrofit2.Call;
import retrofit2.http.GET;
import retrofit2.http.Query;

/**
 * Interfaz para la consulta de libros por ISBN a través de OpenLibrary.
 * La consulta se realiza por medio de una petición GET al servicio,
 * por medio de Retrofit, pasando como parámetro el código ISBN
 * del libro a consultar.
 * Los parámetros adicionales "format" y "jscmd" son fijos. El primero
 * indica que se desea la respuesta en formato JSON, mientras que el
 * segundo obtiene información ampliada del libro, en caso de encontrarse.
 * La información ampliada contiene datos necesarios para la aplicación,
 * tales como la editorial, que no son devueltos por defecto.
 * El JSON devuelto por el servicio es parseado automáticamente a objetos
 * Java.
 */
public interface OpenLibraryApi {

    @GET("books?format=json&jscmd=data")
    Call<Map<String, OpenLibraryBook>> getBookByIsbn(@Query("bibkeys") String isbn);
}
