package edu.uoc.jsev.b4b.data.uimodel;

import android.support.annotation.NonNull;

import java.io.Serializable;
import java.util.Objects;

/**
 * Clase base de la que extienden todas las entidades de interfaz de usuario.
 * Contiene los elementos comunes a todas, tales como su ID o su propietario.
 * La clase implementa la interfaz Comparable, para poder ordenar las entidades
 * en las colecciones.
 */
public class UiEntity implements Serializable, Comparable<UiEntity> {

    // Identificador de la entidad.
    private String id;

    // Identificador del usuario propietario de la entidad.
    private String owner;

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getOwner() {
        return owner;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    /**
     * Sobrescribimos equals para considerar que dos entidades
     * son iguales si sus IDs coinciden.
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        UiEntity uiEntity = (UiEntity) o;
        return Objects.equals(id, uiEntity.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    @Override
    public int compareTo(@NonNull UiEntity uiEntity) {
        return id.compareTo(uiEntity.id);
    }
}
