package edu.uoc.jsev.b4b.data.uimodel;

/**
 * Entidad de interfaz de usuario para los intercambios.
 */
public class UiSwap extends UiEntity {

    // Fecha de alta del intercambio.
    private String fecha;

    // Estado en que se encuentra el intercambio (0:pendiente; 1:aceptado; 2:rechazado)
    private int estado;

    // ID del usuario destinatario de la solicitud de intercambio.
    private String usuarioLibroDestino;

    // Nombre del usuario destinatario de la solicitud de intercambio.
    private String nombreUsuarioLibroDestino;

    // Título del libro solicitado al usuario destinatario del intercambio.
    private String tituloLibroDestino;

    // URL de la foto del libro solicitado al usuario destinatario del intercambio.
    private String fotoLibroDestino;

    // Valoración (1-5) dada por el usuario origen del intercambio al destinatario.
    private int valoracionDestino;

    // ID del usuario que inicia el intercambio.
    private String usuarioLibroOrigen;

    // Nombre del usuario que inicia el intercambio.
    private String nombreUsuarioLibroOrigen;

    // Título del libro solicitado por el destinatario al usuario origen del intercambio.
    private String tituloLibroOrigen;

    // URL de la foto del libro solicitado por el destinatario al usuario origen del intercambio.
    private String fotoLibroOrigen;

    // Valoración (1-5) dada por el usuario destinatario del intercambio al origen.
    private int valoracionOrigen;

    public String getFecha() {
        return fecha;
    }

    public void setFecha(String fecha) {
        this.fecha = fecha;
    }

    public int getEstado() {
        return estado;
    }

    public void setEstado(int estado) {
        this.estado = estado;
    }

    public String getUsuarioLibroDestino() {
        return usuarioLibroDestino;
    }

    public void setUsuarioLibroDestino(String usuarioLibroDestino) {
        this.usuarioLibroDestino = usuarioLibroDestino;
    }

    public String getNombreUsuarioLibroDestino() {
        return nombreUsuarioLibroDestino;
    }

    public void setNombreUsuarioLibroDestino(String nombreUsuarioLibroDestino) {
        this.nombreUsuarioLibroDestino = nombreUsuarioLibroDestino;
    }

    public String getTituloLibroDestino() {
        return tituloLibroDestino;
    }

    public void setTituloLibroDestino(String tituloLibroDestino) {
        this.tituloLibroDestino = tituloLibroDestino;
    }

    public String getFotoLibroDestino() {
        return fotoLibroDestino;
    }

    public void setFotoLibroDestino(String fotoLibroDestino) {
        this.fotoLibroDestino = fotoLibroDestino;
    }

    public int getValoracionDestino() {
        return valoracionDestino;
    }

    public void setValoracionDestino(int valoracionDestino) {
        this.valoracionDestino = valoracionDestino;
    }

    public String getUsuarioLibroOrigen() {
        return usuarioLibroOrigen;
    }

    public void setUsuarioLibroOrigen(String usuarioLibroOrigen) {
        this.usuarioLibroOrigen = usuarioLibroOrigen;
    }

    public String getNombreUsuarioLibroOrigen() {
        return nombreUsuarioLibroOrigen;
    }

    public void setNombreUsuarioLibroOrigen(String nombreUsuarioLibroOrigen) {
        this.nombreUsuarioLibroOrigen = nombreUsuarioLibroOrigen;
    }

    public String getTituloLibroOrigen() {
        return tituloLibroOrigen;
    }

    public void setTituloLibroOrigen(String tituloLibroOrigen) {
        this.tituloLibroOrigen = tituloLibroOrigen;
    }

    public String getFotoLibroOrigen() {
        return fotoLibroOrigen;
    }

    public void setFotoLibroOrigen(String fotoLibroOrigen) {
        this.fotoLibroOrigen = fotoLibroOrigen;
    }

    public int getValoracionOrigen() {
        return valoracionOrigen;
    }

    public void setValoracionOrigen(int valoracionOrigen) {
        this.valoracionOrigen = valoracionOrigen;
    }

    /**
     * Indica si el usuario pasado como parámetro
     * es el que inició el intercambio, es decir,
     * si es el usuario origen del mismo.
     *
     * @param userId ID del usuario.
     * @return true si el usuario es el origen del intercambio,
     * false en caso contrario.
     */
    public boolean isRequested(String userId) {
        return userId.equals(getOwner());
    }

    /**
     * Indica si el usuario pasado como parámetro
     * ha valorado ya al otro usuario implicado
     * en el intercambio.
     *
     * @param userId ID del usuario.
     * @return true, si el usuario ha valorado al otro usuario implicado,
     * false en caso contrario.
     */
    public boolean isRated(String userId) {
        return isRequested(userId) ? (this.valoracionDestino > 0f) : (this.valoracionOrigen > 0f);
    }

    /**
     * Establece la valoración del intercambio que hace
     * el usuario pasado como parámetro.
     *
     * @param rating Valoración realizada por el usuario.
     * @param userId Usuario que realiza la valoración.
     */
    public void setRating(int rating, String userId) {
        if(isRequested(userId)) {
            this.valoracionDestino = rating;
        }
        else {
            this.valoracionOrigen = rating;
        }
    }

    /**
     * Obtiene el nombre del otro usuario participante en el intercambio.
     *
     * @param userId Usuario del que se quiere conocer el otro usuario participante.
     * @return El nombre del otro usuario participante.
     */
    public String getNombreParticipante(String userId) {
        return isRequested(userId) ? this.nombreUsuarioLibroDestino : this.nombreUsuarioLibroOrigen;
    }
}
