package edu.uoc.jsev.b4b.data.uimodel;

import edu.uoc.jsev.b4b.util.MapUtils;

/**
 * Entidad de interfaz de usuario para los usuarios de la aplicación.
 */
public class UiUser extends UiEntity {

    // Nombre del usuario.
    private String nombre;

    // Correo electrónico del usuario.
    private String email;

    // Contraseña de acceso del usuario.
    private String passwd;

    // Número de libros que el usuario tiene registrados en la aplicación.
    private int numeroLibros;

    // Valoración media recibida por el usuario en los intercambios en los que ha participado,
    // bien como origen o bien como destinatario de los mismos.
    private double valoracion;

    // Coordenadas geográficas de la última localización conocida del usuario.
    private double latitud;
    private double longitud;

    public String getNombre() {
        return nombre;
    }

    public void setNombre(String nombre) {
        this.nombre = nombre;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getPasswd() {
        return passwd;
    }

    public void setPassword(String passwd) {
        this.passwd = passwd;
    }

    public int getNumeroLibros() {
        return numeroLibros;
    }

    public void setNumeroLibros(int numeroLibros) {
        this.numeroLibros = numeroLibros;
    }

    public double getValoracion() {
        return valoracion;
    }

    public void setValoracion(double valoracion) {
        this.valoracion = valoracion;
    }

    public double getLatitud() {
        return latitud;
    }

    public void setLatitud(double latitud) {
        this.latitud = latitud;
    }

    public double getLongitud() {
        return longitud;
    }

    public void setLongitud(double longitud) {
        this.longitud = longitud;
    }

    /**
     * Obtiene la distancia en kilómetros del usuario
     * con respecto al punto pasado como parámetro.
     *
     * @param latitud Latitud del punto.
     * @param longitud Longitud del punto.
     * @return La distancia en kilómetros de la última localización
     * conocida del usuario con respecto al punto pasado como parámetro.
     */
    public Double getDistancia(double latitud, double longitud) {
        return MapUtils.distance(latitud, longitud, this.latitud, this.longitud);
    }
}
