package edu.uoc.jsev.b4b.data.uimodel.mapper;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.jsev.b4b.data.backendless.model.Libro;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Mapper para los libros.
 */
public class BookMapper implements Mapper<UiBook, Libro> {

    // Instancia singleton.
    private static BookMapper instance;

    private BookMapper() {

    }

    // Devuelve el singleton.
    public static BookMapper getInstance() {
        if(instance == null) {
            instance = new BookMapper();
        }
        return instance;
    }

    /**
     * @see Mapper#from(Object)
     */
    @Override
    public UiBook from(Libro book) {
        if(book == null) return null;

        UiBook uiBook = new UiBook();
        uiBook.setOwner(book.getOwnerId());
        uiBook.setId(book.getObjectId());
        uiBook.setAutor(book.getAutor());
        uiBook.setEditorial(book.getEditorial());
        uiBook.setEstado(book.getEstado());
        uiBook.setFecha(Utils.formatDate(book.getCreated()));
        uiBook.setFoto(book.getFoto());
        uiBook.setGenero(book.getGenero());
        uiBook.setIdioma(book.getIdioma());
        uiBook.setIsbn(book.getIsbn());
        uiBook.setTitulo(book.getTitulo());

        return uiBook;
    }

    /**
     * @see Mapper#from(UiEntity)
     */
    @Override
    public Libro from(UiBook uiBook) {
        if(uiBook == null) return null;

        Libro book = new Libro();
        book.setObjectId(uiBook.getId());
        book.setAutor(uiBook.getAutor());
        book.setEditorial(uiBook.getEditorial());
        book.setEstado(uiBook.getEstado());
        book.setFoto(uiBook.getFoto());
        book.setGenero(uiBook.getGenero());
        book.setIdioma(uiBook.getIdioma());
        book.setIsbn(uiBook.getIsbn());
        book.setTitulo(uiBook.getTitulo());

        return book;
    }

    /**
     * @see Mapper#fromList(List)
     */
    @Override
    public List<UiBook> fromList(List<Libro> books) {
        List<UiBook> uiBooks = new ArrayList<>();
        for(Libro book : books) {
            uiBooks.add(from(book));
        }
        return uiBooks;
    }
}
