package edu.uoc.jsev.b4b.data.uimodel.mapper;

import java.util.List;

import edu.uoc.jsev.b4b.data.uimodel.UiEntity;

/**
 * Interfaz para mapear entidades de interfaz de usuario
 * a entidades de negocio, y viceversa.
 * Permite traducir de un tipo de entidad a otro, para que
 * las entidades de negocio no lleguen a la vista, ni las
 * de UI lleguen al modelo.
 * Las entidades de interfaz de usuario, normalmente, serán
 * más simples, y contendrán únicamente datos básicos (cadenas,
 * enteros, etcétera), que se puedan mostrar al usuario directamente
 * en la vista.
 * Las entidades de negocio se mapean con los tipos de dato utilizados
 * en el backend.
 *
 * @param <U> Tipo de la entidad de interfaz de usuario.
 * @param <D> Tipo de la entidad de negocio.
 */
public interface Mapper<U extends UiEntity, D> {
    /**
     * Obtiene una entidad de interfaz de usuario
     * a partir de una de negocio.
     *
     * @param dataModel La entidad de negocio.
     * @return La entidad de interfaz de usuario.
     */
    U from(D dataModel);

    /**
     * Obtiene una entidad de negocio
     * a partir de una de interfaz de usuario.
     *
     * @param uiModel La entidad de interfaz de usuario.
     * @return La entidad de negocio.
     */
    D from(U uiModel);

    /**
     * Obtiene una lista de entidades de interfaz de usuario
     * a partir de una lista de entidades de negocio.
     *
     * @param dataModelList Lista de entidades de negocio.
     * @return Lista de entidades de interfaz de usuario.
     */
    List<U> fromList(List<D> dataModelList);
}
