package edu.uoc.jsev.b4b.data.uimodel.mapper;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.jsev.b4b.data.openlibrary.model.OpenLibraryBook;
import edu.uoc.jsev.b4b.data.openlibrary.model.OpenLibraryCover;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Mapper para los libros obtenidos desde OpenLibrary.
 */
public class OpenLibraryMapper implements Mapper<UiBook, OpenLibraryBook> {

    // Instancia singleton.
    private static OpenLibraryMapper instance;

    private OpenLibraryMapper() {

    }

    // Devuelve el singleton.
    public static OpenLibraryMapper getInstance() {
        if(instance == null) {
            instance = new OpenLibraryMapper();
        }
        return instance;
    }

    /**
     * @see Mapper#from(Object)
     */
    @Override
    public UiBook from(OpenLibraryBook book) {
        if(book == null) return null;

        UiBook uiBook = new UiBook();
        uiBook.setTitulo(book.getTitle());
        uiBook.setAutor(Utils.join(book.getAuthors()));
        uiBook.setEditorial(Utils.join(book.getPublishers()));

        OpenLibraryCover bookCover = book.getCover();
        if(bookCover != null) {
            uiBook.setFoto(bookCover.getLarge());
        }

        return uiBook;
    }

    /**
     * Método no implementado, ya que no es necesario.
     * Nunca se van a realizar operaciones de escritura
     * en el servicio OpenLibrary, solo de lectura.
     *
     * @see Mapper#from(UiEntity)
     */
    @Override
    public OpenLibraryBook from(UiBook uiBook) {
        throw new RuntimeException("Not implemented");
    }

    /**
     * @see Mapper#fromList(List)
     */
    @Override
    public List<UiBook> fromList(List<OpenLibraryBook> books) {
        List<UiBook> uiBooks = new ArrayList<>();
        for(OpenLibraryBook book : books) {
            uiBooks.add(from(book));
        }
        return uiBooks;
    }
}
