package edu.uoc.jsev.b4b.data.uimodel.mapper;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.jsev.b4b.data.backendless.model.Intercambio;
import edu.uoc.jsev.b4b.data.backendless.model.Libro;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Mapper para los intercambios.
 */
public class SwapMapper implements Mapper<UiSwap, Intercambio> {

    // Instancia singleton.
    private static SwapMapper instance;

    private SwapMapper() {

    }

    // Devuelve el singleton.
    public static SwapMapper getInstance() {
        if(instance == null) {
            instance = new SwapMapper();
        }
        return instance;
    }

    /**
     * @see Mapper#from(Object)
     */
    @Override
    public UiSwap from(Intercambio swap) {
        if(swap == null) return null;

        UiSwap uiSwap = new UiSwap();
        Libro dstBook = swap.getLibroDestino();
        Libro srcBook = swap.getLibroOrigen();

        uiSwap.setId(swap.getObjectId());
        uiSwap.setOwner(swap.getOwnerId());
        uiSwap.setEstado(swap.getEstado());
        uiSwap.setFecha(Utils.formatDate(swap.getCreated()));
        uiSwap.setFotoLibroDestino(dstBook != null ? dstBook.getFoto() : "");
        uiSwap.setTituloLibroDestino(dstBook != null ? dstBook.getTitulo() : "");
        uiSwap.setUsuarioLibroDestino(dstBook != null ? dstBook.getOwnerId() : "");
        uiSwap.setNombreUsuarioLibroDestino(swap.getUsuarioDestino());
        uiSwap.setValoracionDestino(swap.getValoracionDestino());
        uiSwap.setFotoLibroOrigen(srcBook != null ? srcBook.getFoto() : "");
        uiSwap.setTituloLibroOrigen(srcBook != null ? srcBook.getTitulo() : "");
        uiSwap.setUsuarioLibroOrigen(swap.getOwnerId());
        uiSwap.setNombreUsuarioLibroOrigen(swap.getUsuarioOrigen());
        uiSwap.setValoracionOrigen(swap.getValoracionOrigen());

        return uiSwap;
    }

    /**
     * @see Mapper#from(UiEntity)
     */
    @Override
    public Intercambio from(UiSwap uiSwap) {
        if(uiSwap == null) return null;

        Intercambio swap = new Intercambio();
        swap.setObjectId(uiSwap.getId());
        swap.setEstado(uiSwap.getEstado());
        swap.setUsuarioDestino(uiSwap.getNombreUsuarioLibroDestino());
        swap.setUsuarioOrigen(uiSwap.getNombreUsuarioLibroOrigen());
        swap.setValoracionDestino(uiSwap.getValoracionDestino());
        swap.setValoracionOrigen(uiSwap.getValoracionOrigen());

        return swap;
    }

    /**
     * @see Mapper#fromList(List)
     */
    @Override
    public List<UiSwap> fromList(List<Intercambio> swaps) {
        List<UiSwap> uiSwaps = new ArrayList<>();
        for(Intercambio swap : swaps) {
            uiSwaps.add(from(swap));
        }
        return uiSwaps;
    }
}
