package edu.uoc.jsev.b4b.data.uimodel.mapper;

import com.backendless.BackendlessUser;
import com.backendless.geo.GeoPoint;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.jsev.b4b.data.backendless.model.Usuario;
import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;

/**
 * Mapper para los usuarios de la aplicación.
 */
public class UserMapper implements Mapper<UiUser, BackendlessUser> {

    // Instancia singleton.
    private static UserMapper instance;

    private UserMapper() {

    }

    // Devuelve el singleton.
    public static UserMapper getInstance() {
        if(instance == null) {
            instance = new UserMapper();
        }

        return instance;
    }

    /**
     * @see Mapper#from(Object)
     */
    @Override
    public UiUser from(BackendlessUser user) {
        GeoPoint location = (GeoPoint) user.getProperty(Usuario.LOCATION_KEY);
        UiUser uiUser = new UiUser();
        uiUser.setId(user.getObjectId());
        uiUser.setEmail(user.getEmail());
        uiUser.setPassword(user.getPassword());
        uiUser.setLatitud(location.getLatitude());
        uiUser.setLongitud(location.getLongitude());
        uiUser.setNombre(user.getProperty(Usuario.NAME_KEY).toString());
        uiUser.setNumeroLibros((Integer) user.getProperty(Usuario.BOOKS_NUMBER_KEY));
        uiUser.setValoracion(Double.valueOf(user.getProperty(Usuario.RATING_KEY).toString()));

        return uiUser;
    }

    /**
     * @see Mapper#from(UiEntity)
     */
    @Override
    public BackendlessUser from(UiUser uiUser) {
        BackendlessUser user = new BackendlessUser();
        GeoPoint location = new GeoPoint(uiUser.getLatitud(), uiUser.getLongitud());
        location.addMetadata(Usuario.LOCATION_METADATA_KEY, uiUser.getEmail());
        location.addCategory(Usuario.LOCATION_CATEGORY_KEY);

        user.setEmail(uiUser.getEmail());
        user.setPassword(uiUser.getPasswd());
        user.setProperty(Usuario.NAME_KEY, uiUser.getNombre());
        user.setProperty(Usuario.BOOKS_NUMBER_KEY, uiUser.getNumeroLibros());
        user.setProperty(Usuario.RATING_KEY, uiUser.getValoracion());
        user.setProperty(Usuario.LOCATION_KEY, location);

        return user;
    }

    /**
     * @see Mapper#fromList(List)
     */
    @Override
    public List<UiUser> fromList(List<BackendlessUser> users) {
        List<UiUser> uiUsers = new ArrayList<>();
        for(BackendlessUser user : users) {
            uiUsers.add(from(user));
        }
        return uiUsers;
    }
}
