package edu.uoc.jsev.b4b.ui.base;

/**
 * Interfaz base de la que extienden todas las vistas y presentadores.
 * Ofrece una serie de funcionalidades comunes a todas ellas.
 * Esta interfaz define el contrato que deben cumplir vista y presentador
 * para comunicarse entre ellos.
 */
public interface BaseContract {

    /**
     * Interfaz base para las vistas.
     */
    interface View {
        /**
         * Muestra u oculta un indicador de progreso en la vista.
         * @param show Si su valor es true el indicador se muestra,
         *             si es false, se oculta.
         */
        void showProgress(boolean show);

        /**
         * Oculta el teclado virtual del dispositivo.
         */
        void hideKeyboard();

        /**
         * Muestra por pantalla un mensaje de tipo Toast.
         * @param message Cadena de texto con el mensaje.
         */
        void showToastMessage(String message);

        /**
         * Muestra por pantalla un mensaje de tipo Toast.
         * @param resId Id de recurso con el texto del mensaje.
         */
        void showToastMessage(int resId);
    }

    /**
     * Interfaz base para los presentadores.
     */
    interface Presenter {
        /**
         * Inicializa la pantalla.
         */
        void setup();

        /**
         * El usuario ha pulsado sobre el botón "Up".
         */
        void upButtonPressed();

        /**
         * El usuario ha pulsado sobre el botón "Back".
         */
        void backButtonPressed();
    }
}
