package edu.uoc.jsev.b4b.ui.base;

import android.content.Context;
import android.support.v4.app.Fragment;

import edu.uoc.jsev.b4b.ui.dialog.AlertDialogFragment;

/**
 * Fragmento base del que extienden todos los fragmentos de la aplicación.
 * Implementa una serie de funcionalidades comunes a todos ellos, que delega
 * en la actividad que lo contiene.
 */
public abstract class BaseFragment extends Fragment
        implements BaseContract.View, AlertDialogFragment.AlertDialogListener {

    // Duración por defecto de los mensajes de tipo Snackbar.
    public static final int SNACKBAR_LENGTH_DEFAULT = BaseActivity.SNACKBAR_LENGTH_DEFAULT;

    // Actividad contenedora.
    private BaseActivity mActivity;

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof BaseActivity) {
            mActivity = (BaseActivity) context;
        }
    }

    @Override
    public void onDetach() {
        mActivity = null;
        super.onDetach();
    }

    /**
     * @see BaseContract.View#showProgress(boolean)
     */
    @Override
    public void showProgress(boolean show) {
        if(mActivity != null) {
            mActivity.showProgress(show);
        }
    }

    /**
     * @see BaseContract.View#hideKeyboard()
     */
    @Override
    public void hideKeyboard() {
        if(mActivity != null) {
            mActivity.hideKeyboard();
        }
    }

    /**
     * @see BaseContract.View#showToastMessage(String)
     */
    @Override
    public void showToastMessage(String message) {
        if(mActivity != null) {
            mActivity.showToastMessage(message);
        }
    }

    /**
     * @see BaseContract.View#showToastMessage(int)
     */
    @Override
    public void showToastMessage(int resId) {
        if(mActivity != null) {
            mActivity.showToastMessage(resId);
        }
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogPositiveClick(String, Object...)
     */
    @Override
    public void onDialogPositiveClick(String tag, Object... values) {
        // Por defecto no hacemos nada.
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogNegativeClick(String)
     */
    @Override
    public void onDialogNegativeClick(String tag) {
        // Por defecto no hacemos nada.
    }

    /**
     * @see AlertDialogFragment.AlertDialogListener#onDialogItemClick(String, int)
     */
    @Override
    public void onDialogItemClick(String tag, int which) {
        // Por defecto no hacemos nada.
    }

    /**
     * @see BaseActivity#showAlertDialog(int, int, String)
     */
    protected void showAlertDialog(int titleResId, int messageResId, String tag) {
        if(mActivity != null) {
            mActivity.showAlertDialog(titleResId, messageResId, tag);
        }
    }

    /**
     * @see BaseActivity#showOptionsDialog(int, int[], String)
     */
    protected void showOptionsDialog(int titleResId, int[] items, String tag) {
        if(mActivity != null) {
            mActivity.showOptionsDialog(titleResId, items, tag);
        }
    }

    /**
     * @see BaseActivity#dismissDialog(String)
     */
    protected void dismissDialog(String tag) {
        if(mActivity != null) {
            mActivity.dismissDialog(tag);
        }
    }
}
