package edu.uoc.jsev.b4b.ui.base;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;

/**
 * Interfaz de la que extienden todos los interactores de la aplicación.
 * Ofrece una serie de funcionalidades comunes a todos ellos.
 */
public interface BaseInteractor {
    /**
     * Obtiene el nombre del usuario actualmente autenticado en la aplicación.
     * @return El nombre del usuario actualmente autenticado en la aplicación.
     */
    String getCurrentUserFullName();

    /**
     * Obtiene el email del usuario actualmente autenticado en la aplicación.
     * @return El email del usuario actualmente autenticado en la aplicación.
     */
    String getCurrentUserEmail();

    /**
     * Obtiene el ID del usuario actualmente autenticado en la aplicación.
     * @return El ID del usuario actualmente autenticado en la aplicación.
     */
    String getCurrentUserId();

    /**
     * Obtiene la latitud de la última localización conocida
     * del usuario actualmente autenticado en la aplicación.
     * @return La latitud de la última localización conocida
     * del usuario actualmente autenticado en la aplicación.
     */
    Double getCurrentUserLatitude();

    /**
     * Obtiene la longitud de la última localización conocida
     * del usuario actualmente autenticado en la aplicación.
     * @return La longitud de la última localización conocida
     * del usuario actualmente autenticado en la aplicación.
     */
    Double getCurrentUserLongitude();

    /**
     * Obtiene la lista de géneros literarios disponibles.
     * @param callback Callback al que se devuelve la respuesta asíncrona.
     *                 Si la petición tiene éxito se envía al callback la lista
     *                 de géneros literarios.
     */
    void getAllGenres(ServiceCallback<List<String>> callback);

    /**
     * Obtiene la lista de idiomas disponibles.
     * @param callback Callback al que se devuelve la respuesta asíncrona.
     *                 Si la petición tiene éxito se envía al callback la lista
     *                 de idiomas.
     */
    void getAllLanguages(ServiceCallback<List<String>> callback);

    /**
     * Almacena una imagen en el sistema de archivos remoto del backend.
     *
     * @param picture Imagen a guardar.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback una
     *                 cadena de texto con la ruta de descarga de la imagen.
     */
    void uploadPicture(byte[] picture, ServiceCallback<String> callback);

    /**
     * Envía un mensaje de correo electrónico con el asunto y cuerpo indicados
     * a las direcciones pasadas como parámetro.
     * @param subject Asunto del correo.
     * @param message Cuerpo del correo.
     * @param recipients Direcciones de correo a las que se envía el mensaje.
     */
    void sendEmail(String subject, String message, String... recipients);
}
