package edu.uoc.jsev.b4b.ui.login;

import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.TextInputLayout;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.TextView;

import edu.uoc.jsev.b4b.R;
import edu.uoc.jsev.b4b.ui.base.BaseActivity;
import edu.uoc.jsev.b4b.ui.main.MainActivity;
import edu.uoc.jsev.b4b.ui.registration.RegistrationActivity;
import edu.uoc.jsev.b4b.Injection;

/**
 * Actividad para gestionar la autenticación del usuario en la aplicación.
 */
public class LoginActivity extends BaseActivity implements LoginContract.View {

    // Referencias a las vistas
    private TextInputLayout mEmailLayout;
    private TextInputLayout mPasswordLayout;

    // Presentador
    private LoginContract.Presenter mPresenter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);

        // Inyectamos el presentador.
        mPresenter = Injection.provideLoginPresenter(this);

        // Inicializamos las vistas.
        mEmailLayout = findViewById(R.id.email);

        mPasswordLayout = findViewById(R.id.password);
        mPasswordLayout.getEditText().setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int id, KeyEvent keyEvent) {
                if (id == EditorInfo.IME_ACTION_DONE || id == EditorInfo.IME_NULL) {
                    login();
                    return true;
                }
                return false;
            }
        });

        Button signInButton = findViewById(R.id.sign_in_button);
        signInButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                login();
            }
        });

        TextView notRegisteredLabel = findViewById(R.id.not_registered_label);
        notRegisteredLabel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.registration();
            }
        });

        mProgressView = findViewById(R.id.progress);
    }

    /**
     * @see LoginContract.View#resetValidationErrors()
     */
    @Override
    public void resetValidationErrors() {
        mEmailLayout.setErrorEnabled(false);
        mPasswordLayout.setErrorEnabled(false);
    }

    /**
     * @see LoginContract.View#showEmptyEmailError()
     */
    @Override
    public void showEmptyEmailError() {
        mEmailLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see LoginContract.View#showInvalidEmailError()
     */
    @Override
    public void showInvalidEmailError() {
        mEmailLayout.setError(getString(R.string.error_invalid_email));
    }

    /**
     * @see LoginContract.View#showEmptyPasswordError()
     */
    @Override
    public void showEmptyPasswordError() {
        mPasswordLayout.setError(getString(R.string.error_mandatory_field));
    }

    /**
     * @see LoginContract.View#showLoginError()
     */
    @Override
    public void showLoginError() {
        showToastMessage(R.string.error_incorrect_login);
    }

    /**
     * @see LoginContract.View#navigateToRegistration()
     */
    @Override
    public void navigateToRegistration() {
        startActivity(new Intent(this, RegistrationActivity.class));
    }

    /**
     * @see LoginContract.View#navigateToHome()
     */
    @Override
    public void navigateToHome() {
        startActivity(new Intent(this, MainActivity.class));
        finish();
    }

    /**
     * Método auxiliar para iniciar el proceso de login,
     * enviando los datos introducidos por el usuario al presentador.
     */
    private void login() {
        String email = mEmailLayout.getEditText().getText().toString();
        String password = mPasswordLayout.getEditText().getText().toString();

        mPresenter.login(email, password);
    }
}

