package edu.uoc.jsev.b4b.ui.login;

import edu.uoc.jsev.b4b.ui.base.BaseContract;

/**
 * Contrato entre la vista y el presentador de la pantalla de login.
 */
public interface LoginContract {

    /**
     * Vista de la pantalla de login.
     */
    interface View extends BaseContract.View {
        /**
         * Limpia los errores de validación
         * de los campos de texto de la pantalla.
         */
        void resetValidationErrors();

        /**
         * Muestra un mensaje de error indicando que
         * el campo email está vacío y es obligatorio.
         */
        void showEmptyEmailError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo email tiene un formato incorrecto.
         */
        void showInvalidEmailError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo contraseña está vacío y es obligatorio.
         */
        void showEmptyPasswordError();

        /**
         * Muestra un mensaje de error indicando que
         * las credenciales de acceso no son válidas.
         */
        void showLoginError();

        /**
         * Navega a la pantalla de registro de una nueva cuenta.
         */
        void navigateToRegistration();

        /**
         * Navega a la pantalla principal de la aplicación.
         */
        void navigateToHome();
    }

    /**
     * Presentador de la pantalla de login.
     */
    interface Presenter extends BaseContract.Presenter {
        /**
         * El usuario pulsa sobre el botón de login para autenticarse
         * mediante su correo electrónico y su contraseña.
         * @param email Correo electrónico proporcionado por el usuario.
         * @param password Contraseña proporcionada por el usuario.
         */
        void login(String email, String password);

        /**
         * El usuario pulsa sobre el enlace para navegar
         * hacia la pantalla de registro.
         */
        void registration();
    }
}
