package edu.uoc.jsev.b4b.ui.login;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.ui.base.BasePresenter;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Clase de implementación del presentador de la pantalla de autenticación.
 */
public class LoginPresenter extends BasePresenter implements LoginContract.Presenter {

    // Vista e interactor asociados.
    private LoginContract.View mView;
    private LoginInteractor mInteractor;

    public LoginPresenter(LoginContract.View view, LoginInteractor interactor) {
        this.mView = view;
        this.mInteractor = interactor;
    }

    /**
     * @see LoginContract.Presenter#login(String, String)
     */
    @Override
    public void login(String email, String password) {
        boolean cancelLogin = false;
        mView.resetValidationErrors();

        // Validamos la información introducida antes de hacer login:

        // El email es obligatorio.
        if (Utils.isEmptyString(password)) {
            mView.showEmptyPasswordError();
            cancelLogin = true;
        }

        // El email es obligatorio y debe tener el formato correcto.
        if (Utils.isEmptyString(email)) {
            mView.showEmptyEmailError();
            cancelLogin = true;
        }
        else if (!Utils.isEmailValid(email)) {
            mView.showInvalidEmailError();
            cancelLogin = true;
        }

        // Si los datos son correctos, intentamos el login.
        if(!cancelLogin) {
            mView.hideKeyboard();
            doLogin(email, password);
        }
    }

    /**
     * @see LoginContract.Presenter#registration()
     */
    @Override
    public void registration() {
        mView.navigateToRegistration();
    }

    private void doLogin(String email, String password) {
        mView.showProgress(true);
        mInteractor.login(email, password, new ServiceCallback<Void>() {
            @Override
            public void onSuccess(Void response) {
                // Si el login es correcto pasamos a la pantalla principal.
                mView.showProgress(false);
                mView.navigateToHome();
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si el login no es correcto mostramos un mensaje al usuario.
                mView.showProgress(false);
                mView.showLoginError();
            }
        });
    }
}
