package edu.uoc.jsev.b4b.ui.main;

import edu.uoc.jsev.b4b.ui.base.BaseContract;

/**
 * Contrato entre la vista y el presentador de la pantalla principal.
 */
public interface MainContract {

    /**
     * Vista de la pantalla principal.
     */
    interface View extends BaseContract.View {
        /**
         * Ejecuta la acción asociada al botón flotante.
         */
        void fabButtonClicked();

        /**
         * Establece el nombre del usuario autenticado
         * en la cabecera del menú lateral de navegación
         * @param fullName El nombre del usuario autenticado.
         */
        void setCurrentUserFullName(String fullName);

        /**
         * Establece el correo electrónico del usuario autenticado
         * en la cabecera del menú lateral de navegación
         * @param email El correo electrónico del usuario autenticado.
         */
        void setCurrentUserEmail(String email);

        /**
         * Ejecuta la acción estándar asociada al botón "Back"
         * (retroceder a la pantalla anterior).
         */
        void backButtonNormalAction();

        /**
         * Muestra la pantalla con el mapa.
         */
        void showMap();

        /**
         * Muestra la pantalla con el listado de libros.
         */
        void showMyBooks();

        /**
         * Muestra la pantalla con el listado de intercambios.
         */
        void showSwaps();

        /**
         * Muestra un diálogo para que el usuario
         * confirme si desea desconectarse de la aplicación.
         */
        void showLogoutDialog();

        /**
         * Muestra un mensaje de error indicando que
         * el proceso de desconexión no se ha podido
         * realizar correctamente.
         */
        void showLogoutError();

        /**
         * Cierra el diálogo de confirmación de logout.
         */
        void dismissLogoutDialog();

        /**
         * Indica si el menú de navegación lateral está abierto.
         * @return true si está abierto, false si está cerrado.
         */
        boolean isNavigationDrawerOpen();

        /**
         * Cierra el menú de navegación lateral.
         */
        void closeNavigationDrawer();

        /**
         * Navega a la pantalla de autenticación.
         */
        void navigateToLogin();
    }

    /**
     * Presentador de la pantalla principal.
     */
    interface Presenter extends BaseContract.Presenter {
        /**
         * El usuario pulsa sobre el botón flotante,
         * en el que se encuentra la acción principal.
         */
        void mainActionClicked();

        /**
         * El usuario pulsa sobre la opción de menú "Cerca de mí".
         */
        void mapOptionClicked();

        /**
         * El usuario pulsa sobre la opción de menú "Mis libros".
         */
        void myBooksOptionClicked();

        /**
         * El usuario pulsa sobre la opción de menú "Intercambios".
         */
        void swapsOptionClicked();

        /**
         * El usuario pulsa sobre la opción de menú "Salir".
         */
        void logoutOptionClicked();

        /**
         * El usuario confirma la desconexión,
         * al pulsar sobre el botón "Aceptar"
         * del diálogo de logout.
         */
        void logout();

        /**
         * El usuario cancela la desconexión,
         * al pulsar sobre el botón "Cancelar"
         * del diálogo de logout.
         */
        void logoutCanceled();
    }
}
