package edu.uoc.jsev.b4b.ui.main;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.ui.base.BasePresenter;

/**
 * Clase de implementación del presentador de la pantalla principal.
 */
public class MainPresenter extends BasePresenter implements MainContract.Presenter {

    // Vista e interactor asociados.
    private MainContract.View mView;
    private MainInteractor mInteractor;

    public MainPresenter(MainContract.View view, MainInteractor interactor) {
        this.mView = view;
        this.mInteractor = interactor;
    }

    /**
     * @see MainContract.Presenter#setup()
     */
    @Override
    public void setup() {
        // Mostramos los datos del usuario en el menú de navegación
        // y cargamos el fragment por defecto, el del mapa.
        mView.setCurrentUserFullName(mInteractor.getCurrentUserFullName());
        mView.setCurrentUserEmail(mInteractor.getCurrentUserEmail());
        mView.showMap();
    }

    /**
     * @see MainContract.Presenter#backButtonPressed()
     */
    @Override
    public void backButtonPressed() {
        // Si el menú de navegación está abierto al pulsar "Back" lo cerramos.
        if(mView.isNavigationDrawerOpen()) {
            mView.closeNavigationDrawer();
        }
        // Si está cerrado, volvemos a la pantalla anterior.
        else {
            mView.backButtonNormalAction();
        }
    }

    /**
     * @see MainContract.Presenter#mainActionClicked()
     */
    @Override
    public void mainActionClicked() {
        mView.fabButtonClicked();
    }

    /**
     * @see MainContract.Presenter#myBooksOptionClicked()
     */
    @Override
    public void myBooksOptionClicked() {
        // Cerramos el menú y cargamos la lista de libros.
        mView.closeNavigationDrawer();
        mView.showMyBooks();
    }

    /**
     * @see MainContract.Presenter#mapOptionClicked()
     */
    @Override
    public void mapOptionClicked() {
        // Cerramos el menú y cargamos el mapa.
        mView.closeNavigationDrawer();
        mView.showMap();
    }

    /**
     * @see MainContract.Presenter#swapsOptionClicked()
     */
    @Override
    public void swapsOptionClicked() {
        // Cerramos el menú y cargamos la lista de intercambios.
        mView.closeNavigationDrawer();
        mView.showSwaps();
    }

    /**
     * @see MainContract.Presenter#logoutOptionClicked()
     */
    @Override
    public void logoutOptionClicked() {
        // Cerramos el menú y mostramos la confirmación de desconexión.
        mView.closeNavigationDrawer();
        mView.showLogoutDialog();
    }

    /**
     * @see MainContract.Presenter#logout()
     */
    @Override
    public void logout() {
        mView.showProgress(true);
        mInteractor.logout(new ServiceCallback<Void>() {
            @Override
            public void onSuccess(Void response) {
                // Si el logout es correcto pasamos a la pantalla de login.
                mView.showProgress(false);
                mView.navigateToLogin();
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si el logout no es correcto mostramos un mensaje al usuario.
                mView.showProgress(false);
                mView.showLogoutError();
            }
        });
    }

    /**
     * @see MainContract.Presenter#logoutCanceled()
     */
    @Override
    public void logoutCanceled() {
        // Cerramos la ventana del diálogo de confirmación de logout.
        mView.dismissLogoutDialog();
    }
}
