package edu.uoc.jsev.b4b.ui.map;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BaseInteractor;

/**
 * Interfaz del interactor de la pantalla del mapa.
 */
public interface MapInteractor extends BaseInteractor {
    /**
     * Actualiza la última localización conocida del usuario.
     * @param latitude Latitud de la nueva localización.
     * @param longitude Longitud de la nueva localización.
     */
    void updateUserLastKnownLocation(double latitude, double longitude);

    /**
     * Devuelve la lista de usuarios que comparten algún libro
     * que verifique los criterios indicados en los parámetros.
     * Si se especifican varios criterios, la búsqueda se realiza
     * mediante un operador lógico "AND".
     * @param title Búsqueda de libros por título.
     * @param author Búsqueda de libros por autor.
     * @param genre Búsqueda de libros por genero literario.
     * @param distance Búsqueda de libros por distancia con el usuario
     *                 propietario del libro. Se devolverán los usuarios
     *                 que se encuentren en un radio igual o inferior a la
     *                 distancia especificada (en kilómetros).
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback la lista
     *                 de usuarios que comparten algún libro que verifique los
     *                 criterios de búsqueda.
     */
    void findUsers(String title, String author, String genre, int distance, ServiceCallback<List<UiUser>> callback);
}
