package edu.uoc.jsev.b4b.ui.map;

import com.backendless.BackendlessUser;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.service.CommonsService;
import edu.uoc.jsev.b4b.data.backendless.service.SearchService;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BaseInteractorImpl;

/**
 * Clase de implementación del interactor de la pantalla del mapa.
 */
public class MapInteractorImpl extends BaseInteractorImpl implements MapInteractor {

    // Instancia singleton.
    private static MapInteractor instance;

    // Servicio para realizar la búsqueda de usuarios/libros.
    private SearchService mSearchService;

    private MapInteractorImpl(SearchService searchService, CommonsService commonsService) {
        super(searchService, commonsService);
        this.mSearchService = searchService;
    }

    // Devuelve el singleton.
    public static MapInteractor getInstance(SearchService searchService, CommonsService commonsService) {
        if (instance == null) {
            instance = new MapInteractorImpl(searchService, commonsService);
        }
        return instance;
    }

    /**
     * @see MapInteractor#updateUserLastKnownLocation(double, double)
     */
    @Override
    public void updateUserLastKnownLocation(double latitude, double longitude) {
        mSearchService.updateUserLastKnownLocation(latitude, longitude);
    }

    /**
     * @see MapInteractor#findUsers(String, String, String, int, ServiceCallback)
     */
    @Override
    public void findUsers(String title, String author, String genre, int distance, final ServiceCallback<List<UiUser>> callback) {
        mSearchService.findUsers(title, author, genre, distance, new ServiceCallback<List<BackendlessUser>>() {
            @Override
            public void onSuccess(List<BackendlessUser> response) {
                callback.onSuccess(userMapper.fromList(response));
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }
}
