package edu.uoc.jsev.b4b.ui.map.userdetail;

import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;

/**
 * Contrato entre la vista y el presentador
 * de la pantalla de detalle de usuario.
 */
public interface UserDetailContract {

    /**
     * Vista de la pantalla de detalle de usuario.
     */
    interface View extends RecyclerViewContract.View {
        /**
         * Devuelve los datos del usuario a mostar en pantalla.
         * @return El usuario a mostrar.
         */
        UiUser getUserData();

        /**
         * Devuelve los parámetros adicionales
         * que han sido enviados a la actividad.
         * @return Los parámetros adicionales de la actividad.
         */
        Object[] getParams();

        /**
         * Establece en la pantalla el nombre del usuario.
         * @param name El nombre del usuario.
         */
        void setName(String name);

        /**
         * Establece en la pantalla el número de libros
         * compartidos por el usuario.
         * @param booksNumber El número de libros compartidos
         *                    por el usuario.
         */
        void setBooksNumber(String booksNumber);

        /**
         * Establece en la pantalla la distancia
         * a la que se encuentra el usuario (en kms).
         * @param distance La distancia a la que se encuentra
         *                 el usuario (en kms).
         */
        void setDistance(String distance);

        /**
         * Establece en la pantalla la valoración
         * media de los intercambios en los que
         * ha participado el usuario.
         * @param rating La valoración media.
         */
        void setRating(String rating);

        /**
         * Navega a la pantalla del mapa.
         */
        void navigateToMap();

        /**
         * Navega a la pantalla de detalle de intercambio.
         */
        void navigateToSwapDetail();
    }

    /**
     * Presentador de la pantalla de detalle de usuario.
     * No incluye ningún método adicional, pero se crea por mantener
     * la misma jerarquía que el resto de pantallas.
     */
    interface Presenter extends RecyclerViewContract.Presenter {

    }
}
