package edu.uoc.jsev.b4b.ui.map.userdetail;

import edu.uoc.jsev.b4b.data.uimodel.UiEntity;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;
import edu.uoc.jsev.b4b.ui.mybooks.MyBooksRecyclerViewPresenter;
import edu.uoc.jsev.b4b.util.Utils;

/**
 * Clase de implementación del presentador de la pantalla de detalle de usuario.
 */
public class UserDetailPresenter extends MyBooksRecyclerViewPresenter implements UserDetailContract.Presenter {

    // Datos del usuario mostrado en la pantalla.
    private UiUser mUserData;

    // En caso de venir desde la pantalla de detalle de intercambio,
    // almacena el id del intercambio que se aceptará en caso de
    // que finalmente se acabe solicitando el libro en la pantalla
    // de detalle de libro.
    private String mSwapId;

    // Vista asociada.
    private UserDetailContract.View mView;

    public UserDetailPresenter(UserDetailContract.View view, UserDetailInteractor interactor) {
        super(view, interactor);
        this.mView = view;
    }

    /**
     * @see UserDetailContract.Presenter#setup()
     */
    @Override
    public void setup() {
        // Obtenemos los datos del usuario,
        // solicitándolos a la vista.
        if(mUserData == null) {
            mUserData = mView.getUserData();
            mSwapId = (String) mView.getParams()[0];
        }

        // Mostramos los datos por pantalla.
        mView.setName(mUserData.getNombre());
        mView.setBooksNumber(String.valueOf(mUserData.getNumeroLibros()));
        mView.setRating(Utils.formatDoubleIfGreaterThan1(mUserData.getValoracion()));
        mView.setDistance(Utils.formatDouble(
                mUserData.getDistancia(mInteractor.getCurrentUserLatitude(), mInteractor.getCurrentUserLongitude())));
    }

    /**
     * @see UserDetailContract.Presenter#loadData()
     */
    @Override
    public void loadData() {
        // Obtenemos los libros del usuario
        // y los añadimos al listado.
        loadBooks(mUserData.getId());
    }

    /**
     * @see RecyclerViewContract.Presenter#dataClicked(UiEntity)
     */
    @Override
    public void dataClicked(UiEntity data) {
        // Al pulsar sobre un elemento del listado
        // navegamos al detalle del libro.
        // Sobrescribimos el comportamiento por defecto
        // para añadir parámetros adicionales a la actividad
        // destino (el id de intercambio).
        mView.navigateToDetail(data, mSwapId);
    }

    /**
     * @see UserDetailContract.Presenter#backButtonPressed()
     */
    @Override
    public void backButtonPressed() {
        // Establecemos la navegación a la pantalla anterior
        // en función de la procedencia.
        if(Utils.isEmptyString(mSwapId)) {
            // Venimos del mapa => Volvemos al mapa.
            mView.navigateToMap();
        }
        else {
            // Venimos de la pantalla de detalle de intercambio
            // => Volvemos al detalle de intercambio.
            mView.navigateToSwapDetail();
        }
    }
}
