package edu.uoc.jsev.b4b.ui.mybooks;

import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;

/**
 * Contrato entre la vista y el presentador
 * de la pantalla de listado de libros.
 */
public interface MyBooksContract {

    /**
     * Vista de la pantalla de listado de libros.
     */
    interface View extends RecyclerViewContract.View {
        /**
         * Establece el título de la pantalla.
         */
        void setTitle();

        /**
         * Muestra/oculta el botón flotante.
         * @param show true para mostrarlo, false para ocultarlo.
         */
        void showFabButton(boolean show);

        /**
         * Navega hacia la pantalla para dar
         * de alta un nuevo libro.
         */
        void navigateToNewBook();
    }

    /**
     * Presentador de la pantalla de listado de libros.
     */
    interface Presenter extends RecyclerViewContract.Presenter {
        /**
         * Añade un nuevo libro al listado.
         * @param book El libro a añadir.
         */
        void addData(UiBook book);

        /**
         * Elimina un libro del listado.
         * @param book El libro a eliminar.
         */
        void removeData(UiBook book);

        /**
         * El usuario ha pulsado sobre la acción principal,
         * situada en el botón flotante.
         */
        void mainActionClicked();
    }
}
