package edu.uoc.jsev.b4b.ui.mybooks;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Libro;
import edu.uoc.jsev.b4b.data.backendless.service.BookService;
import edu.uoc.jsev.b4b.data.backendless.service.CommonsService;
import edu.uoc.jsev.b4b.data.backendless.service.UserService;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.ui.base.BaseBooksInteractorImpl;

/**
 * Clase de implementación del interactor de la pantalla del listado de libros.
 */
public class MyBooksInteractorImpl extends BaseBooksInteractorImpl implements MyBooksInteractor {

    // Instancia singleton.
    private static MyBooksInteractor instance;

    protected MyBooksInteractorImpl(UserService userService,
                                  CommonsService commonsService,
                                  BookService bookService) {
        super(userService, commonsService, bookService);
    }

    // Devuelve el singleton.
    public static MyBooksInteractor getInstance(UserService userService,
                                                CommonsService commonsService,
                                                BookService bookService) {
        if (instance == null) {
            instance = new MyBooksInteractorImpl(userService, commonsService, bookService);
        }
        return instance;
    }

    /**
     * @see MyBooksInteractor#getUserBooks(String, ServiceCallback)
     */
    @Override
    public void getUserBooks(String userId, final ServiceCallback<List<UiBook>> callback) {
        mBookService.getUserBooks(userId, new ServiceCallback<List<Libro>>() {
            @Override
            public void onSuccess(List<Libro> response) {
                callback.onSuccess(bookMapper.fromList(response));
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }
}
