package edu.uoc.jsev.b4b.ui.mybooks;

import java.util.Collections;
import java.util.List;

import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;

/**
 * Clase de implementación del presentador de la pantalla del listado de libros.
 */
public class MyBooksPresenter extends MyBooksRecyclerViewPresenter implements MyBooksContract.Presenter {

    // Vista asociada.
    private MyBooksContract.View mView;

    public MyBooksPresenter(MyBooksContract.View view, MyBooksInteractor interactor) {
        super(view, interactor);
        this.mView = view;
    }

    /**
     * @see MyBooksContract.Presenter#setup()
     */
    @Override
    public void setup() {
        // Establecemos el título de la actividad contenedora
        // y mostramos el botón flotante, para añadir un nuevo libro.
        mView.setTitle();
        mView.showFabButton(true);
    }

    /**
     * @see RecyclerViewContract.Presenter#loadData()
     */
    @Override
    public void loadData() {
        // Obtenemos los libros del usuario
        // y los añadimos al listado.
        loadBooks(mInteractor.getCurrentUserId());
    }

    /**
     * @see MyBooksContract.Presenter#addData(UiBook)
     */
    @Override
    public void addData(UiBook book) {
        // Añadimos el libro y ordenamos la lista,
        // para que aparezca ordenada alfabéticamente.
        // Refrescamos el listado, para que se muestre
        // el nuevo elemento.
        List<UiBook> books = (List<UiBook>) mDataList;
        books.add(book);
        Collections.sort(books);

        mView.refreshData();
    }

    /**
     * @see MyBooksContract.Presenter#removeData(UiBook)
     */
    @Override
    public void removeData(UiBook book) {
        // Eliminamos el libro y refrescamos
        // el listado para que se reflejen los cambios.
        mDataList.remove(book);
        mView.refreshData();
    }

    /**
     * @see MyBooksContract.Presenter#mainActionClicked()
     */
    @Override
    public void mainActionClicked() {
        // Al pulsar sobre el FAB navegamos
        // hacia la pantalla de nuevo libro.
        mView.navigateToNewBook();
    }
}
