package edu.uoc.jsev.b4b.ui.mybooks;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewPresenter;

/**
 * Implementación base de la que extienden las implementaciones
 * de presentadores de aquellas pantallas que muestran un listado de libros,
 * es decir, "Mis libros" y "Detalle de usuario".
 * Implementa las funcionalidades comunes a todos ellos.
 */
public abstract class MyBooksRecyclerViewPresenter extends RecyclerViewPresenter {

    // Interactor asociado.
    protected MyBooksInteractor mInteractor;

    public MyBooksRecyclerViewPresenter(RecyclerViewContract.View view, MyBooksInteractor interactor) {
        super(view);
        this.mInteractor = interactor;
    }

    /**
     * @see RecyclerViewContract.Presenter#bindDataAtPosition(RecyclerViewContract.RowView, int)
     */
    @Override
    public void bindDataAtPosition(RecyclerViewContract.RowView rowView, int position) {
        // Cargamos los datos del libro en el elemento del listado
        // situado en la posición indicada por el parámetro "position".
        UiBook data = (UiBook) mDataList.get(position);
        rowView.setData(data);
        rowView.setTitle(data.getTitulo());
        rowView.setSubtitle(data.getAutor());
        rowView.setExtraInfo(data.getFecha());
        rowView.setImage(data.getFoto());
    }

    /**
     * Método auxiliar, utilizable por las subclases,
     * que obtiene los los libros del usuario pasado
     * como parámetro.
     * @param userId Id del usuario cuyos libros se desea obtener.
     */
    protected void loadBooks(String userId) {
        mView.showProgress(true);
        mInteractor.getUserBooks(userId, new ServiceCallback<List<UiBook>>() {
            @Override
            public void onSuccess(List<UiBook> response) {
                // Si los libros se obtienen correctamente
                // los añadimos al listado y los mostramos.
                mView.showProgress(false);
                mDataList = response;
                mView.refreshData();
            }

            @Override
            public void onError(ServiceFault fault) {
                // Si se produce algún error al obtener los libros
                // mostramos un mensaje al usuario.
                mView.showProgress(false);
                mView.showLoadDataError();
            }
        });
    }
}
