package edu.uoc.jsev.b4b.ui.mybooks.bookdetail;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.base.BaseBooksInteractor;

/**
 * Interfaz del interactor de la pantalla de detalle de libro.
 */
public interface BookDetailInteractor extends BaseBooksInteractor {
    /**
     * Elimina el libro pasado como parámetro.
     * Antes de eliminarlo se comprueba que no se esté utilizando
     * en algún intercambio que aún esté pendiente. En ese caso,
     * no se permite su borrado.
     * El método también actualiza el contador del número de libros
     * del usuario propietario, decremetándolo en uno.
     * @param book El libro a eliminar.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito y el libro se ha eliminado
     *                 se devuelve al callback true. Si no se puede eliminar
     *                 porque el libro está siendo usado en algún intercambio
     *                 pendiente se devuelve al callback false. Si se produce
     *                 algún error en la eliminación se informa al callback
     *                 por medio de su método onError.
     */
    void deleteBook(UiBook book, ServiceCallback<Boolean> callback);

    /**
     * Inicia un nuevo intercambio, solicitando el libro pasado como parámetro
     * al usuario pasado como parámetro. Antes de solicitarlo se comprueba que
     * no se haya solicitado ya y esté aún pendiente de aceptar por el otro usuario.
     * Si el intercambio se registra correctamente, se envía un correo
     * electrónico al usuario destinatario del intercambio para informarle de que
     * ha recibido una nueva solicitud de intercambio.
     * @param userId Id del usuario destinatario del intercambio.
     * @param bookId Id del libro solicitado al usuario destinatario.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se devuelve en el callback
     *                 el intercambio creado.
     */
    void requestNewSwap(String userId, String bookId, ServiceCallback<UiSwap> callback);

    /**
     * Acepta un intercambio recibido por el usuario, mediante la
     * solicitud al usuario origen del mismo de uno de sus libros.
     * Cuando esto ocurre, implícitamente, se establece el estado
     * del intercambio a aceptado.
     * Antes de solicitarlo se comprueba que no se haya solicitado
     * ya y esté aún pendiente de aceptar por el otro usuario.
     * Si la actualización se registra correctamente, se envía un correo
     * electrónico a ambos usuarios para informarles de que el intercambio
     * ha sido aceptado y para facilitarles sus respectivas direcciones de email,
     * para que se puedan poner en contacto entre ellos.
     * @param swapId Id del intercambio aceptado.
     * @param bookId Id del libro solicitado al usuario origen.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se devuelve en el callback
     *                 el intercambio con los datos actualizados.
     */
    void acceptSwap(String swapId, String bookId, ServiceCallback<UiSwap> callback);
}
