package edu.uoc.jsev.b4b.ui.mybooks.newbook;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.uimodel.UiBook;
import edu.uoc.jsev.b4b.ui.base.BaseBooksInteractor;

/**
 * Interfaz del interactor de la pantalla de nuevo libro.
 */
public interface NewBookInteractor extends BaseBooksInteractor {
    /**
     * Obtiene los datos de un libro buscándolo por su ISBN.
     * @param isbn ISBN del libro a consultar.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el libro
     *                 encontrado. Si el libro no se encuentra se devuelve al
     *                 callback un objeto nulo.
     */
    void getBookByIsbn(String isbn, ServiceCallback<UiBook> callback);

    /**
     * Registra un nuevo libro en el backend asociado al usuario.
     * Si se incluye en los datos el ISBN del libro y dicho código
     * ya está registrado por el usuario, el libro no se inserta.
     * El método actualiza también el contador de libros del usuario,
     * incrementándolo en uno.
     * @param book Los datos del nuevo libro.
     * @param picture La fotografía a asociar al libro.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el libro
     *                 insertado.
     */
    void addBook(UiBook book, byte[] picture, ServiceCallback<UiBook> callback);
}
