package edu.uoc.jsev.b4b.ui.registration;

import edu.uoc.jsev.b4b.ui.base.BaseContract;

/**
 * Contrato entre la vista y el presentador de la pantalla de registro.
 */
public interface RegistrationContract {

    /**
     * Vista de la pantalla de registro.
     */
    interface View extends BaseContract.View {
        /**
         * Limpia los errores de validación
         * de los campos de texto de la pantalla.
         */
        void resetValidationErrors();

        /**
         * Muestra un mensaje de error indicando que
         * el campo nombre está vacío y es obligatorio.
         */
        void showEmptyFullNameError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo localización está vacío y es obligatorio.
         */
        void showEmptyLocationError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo email está vacío y es obligatorio.
         */
        void showEmptyEmailError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo email tiene un formato incorrecto.
         */
        void showInvalidEmailError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo contraseña está vacío y es obligatorio.
         */
        void showEmptyPasswordError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo contraseña no tiene la longitud
         * mínima requerida.
         */
        void showInvalidPasswordError();

        /**
         * Muestra un mensaje de error indicando que
         * el campo contraseña y su campo de confirmación
         * no coinciden.
         */
        void showPasswordsDontMatchError();

        /**
         * Muestra un mensaje de error indicando que
         * el registro de la cuenta no se ha podido
         * realizar correctamente.
         */
        void showRegistrationError();

        /**
         * Muestra un mensaje de error indicando que
         * ha habido un problema al realizar la búsqueda
         * de la localización del usuario.
         */
        void showLocationSearchError();

        /**
         * Navega a la pantalla de autenticación.
         */
        void navigateToLogin();

        /**
         * Navega a la pantalla principal de la aplicación.
         */
        void navigateToHome();
    }

    /**
     * Presentador de la pantalla de registro.
     */
    interface Presenter extends BaseContract.Presenter {
        /**
         * El usuario ha seleccionado una localización.
         * @param latitude La latitud de la localización seleccionada.
         * @param longitude La longitud de la localización seleccionada.
         */
        void locationSelected(double latitude, double longitude);

        /**
         * Se ha producido un error cuando el usuario
         * intentaba seleccionar una localización.
         */
        void locationSearchError();

        /**
         * El usuario pulsa sobre el botón de registro para crear la cuenta.
         * @param fullName Nombre proporcionado por el usuario.
         * @param email Correo electrónico proporcionado por el usuario.
         * @param password Contraseña proporcionada por el usuario.
         * @param repeatPassword Confirmación de la contraseña proporcionada por el usuario.
         * @param location Nombre de la localización proporcionada por el usuario.
         */
        void createAccount(String fullName, String email, String password, String repeatPassword, String location);

        /**
         * El usuario pulsa sobre el enlace para navegar
         * hacia la pantalla de autenticación.
         */
        void login();
    }
}
