package edu.uoc.jsev.b4b.ui.registration;

import com.backendless.BackendlessUser;
import com.backendless.geo.GeoPoint;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Usuario;
import edu.uoc.jsev.b4b.data.backendless.service.CommonsService;
import edu.uoc.jsev.b4b.data.backendless.service.UserService;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BaseInteractorImpl;

/**
 * Clase de implementación del interactor de la pantalla de registro.
 */
public class RegistrationInteractorImpl extends BaseInteractorImpl implements RegistrationInteractor {

    // Instancia singleton.
    private static RegistrationInteractor instance;

    private RegistrationInteractorImpl(UserService userService, CommonsService commonsService) {
        super(userService, commonsService);
    }

    // Devuelve el singleton.
    public static RegistrationInteractor getInstance(UserService userService, CommonsService commonsService) {
        if (instance == null) {
            instance = new RegistrationInteractorImpl(userService, commonsService);
        }
        return instance;
    }

    /**
     * @see RegistrationInteractor#createAccount(UiUser, ServiceCallback)
     */
    @Override
    public void createAccount(final UiUser user, final ServiceCallback<Void> callback) {
        BackendlessUser backendlessUser = userMapper.from(user);
        final GeoPoint location = (GeoPoint) backendlessUser.getProperty(Usuario.LOCATION_KEY);
        backendlessUser.removeProperty(Usuario.LOCATION_KEY);

        // Registramos la cuenta de usuario.
        mUserService.registerAccount(backendlessUser, new ServiceCallback<BackendlessUser>() {
            @Override
            public void onSuccess(final BackendlessUser userResponse) {
                // Una vez registrada creamos su ubicación inicial y la relacionamos con la cuenta.
                userResponse.setProperty(Usuario.LOCATION_KEY, location);
                mUserService.addInitialLocation(userResponse, new ServiceCallback<Void>() {
                    @Override
                    public void onSuccess(Void response) {
                        // Si el proceso es correcto, autenticamos
                        // al usuario y accedemos a la aplicación.
                        mUserService.login(userResponse.getEmail(), userResponse.getPassword(), new ServiceCallback<Void>() {
                            @Override
                            public void onSuccess(Void response) {
                                callback.onSuccess(null);
                            }

                            @Override
                            public void onError(ServiceFault fault) {
                                callback.onError(fault);
                            }
                        });
                    }

                    @Override
                    public void onError(ServiceFault fault) {
                        callback.onError(fault);
                    }
                });
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }
}
