package edu.uoc.jsev.b4b.ui.swaps;

import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.adapter.recyclerview.RecyclerViewContract;

/**
 * Contrato entre la vista y el presentador
 * de la pantalla de listado de intercambios.
 */
public interface SwapsContract {

    /**
     * Vista de la pantalla de listado de intercambios.
     */
    interface View extends RecyclerViewContract.View {
        /**
         * Establece el título de la pantalla.
         */
        void setTitle();

        /**
         * Muestra/oculta el botón flotante.
         * @param show true para mostrarlo, false para ocultarlo.
         */
        void showFabButton(boolean show);

        /**
         * Devuelve el identificador de recurso del icono
         * de intercambio solicitado (flecha arriba).
         * @return El id de recurso del icono.
         */
        int getRequestedSwapIcon();

        /**
         * Devuelve el identificador de recurso del icono
         * de solicitud de intercambio recibida (flecha abajo).
         * @return El id de recurso del icono.
         */
        int getDestinationSwapIcon();

        /**
         * Devuelve el identificador de recurso del color
         * de intercambio en estado pendiente (naranja).
         * @return El id de recurso del color.
         */
        int getPendingSwapColor();

        /**
         * Devuelve el identificador de recurso del color
         * de intercambio en estado cancelado (rojo).
         * @return El id de recurso del color.
         */
        int getCanceledSwapColor();

        /**
         * Devuelve el identificador de recurso del color
         * de intercambio en estado aceptado (verde).
         * @return El id de recurso del color.
         */
        int getAcceptedSwapColor();

        /**
         * Devuelve la cadena "Solicitado por USUARIO".
         * @param userName El nombre de usuario a incluir
         *                 en la cadena devuelta.
         * @return La cadena "Solicitado por USUARIO".
         */
        String getRequestedByLabel(String userName);

        /**
         * Devuelve la cadena "Solicitado a USUARIO".
         * @param userName El nombre de usuario a incluir
         *                 en la cadena devuelta.
         * @return La cadena "Solicitado a USUARIO".
         */
        String getRequestedToLabel(String userName);
    }

    /**
     * Presentador de la pantalla de listado de intercambios.
     */
    interface Presenter extends RecyclerViewContract.Presenter {
        /**
         * Añade un nuevo intercambio al listado.
         * @param swapData El intercambio a añadir.
         */
        void addData(UiSwap swapData);

        /**
         * Actualiza los datos de un intercambio del listado.
         * @param swapData El intercambio a actualizar.
         */
        void updateData(UiSwap swapData);

        /**
         * El usuario ha pulsado sobre la acción principal,
         * situada en el botón flotante.
         */
        void mainActionClicked();
    }
}
