package edu.uoc.jsev.b4b.ui.swaps;

import java.util.List;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.ServiceFault;
import edu.uoc.jsev.b4b.data.backendless.model.Intercambio;
import edu.uoc.jsev.b4b.data.backendless.service.CommonsService;
import edu.uoc.jsev.b4b.data.backendless.service.SwapService;
import edu.uoc.jsev.b4b.data.backendless.service.UserService;
import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.ui.base.BaseSwapsInteractorImpl;

/**
 * Clase de implementación del interactor de la pantalla del listado de intercambios.
 */
public class SwapsInteractorImpl extends BaseSwapsInteractorImpl implements SwapsInteractor {

    // Instancia singleton.
    private static SwapsInteractor instance;

    private SwapsInteractorImpl(UserService userService,
                                CommonsService commonsService,
                                SwapService swapService) {
        super(userService, commonsService, swapService);
    }

    // Devuelve el singleton.
    public static SwapsInteractor getInstance(UserService userService,
                                                  CommonsService commonsService,
                                                  SwapService swapService) {
        if (instance == null) {
            instance = new SwapsInteractorImpl(userService, commonsService, swapService);
        }
        return instance;
    }

    /**
     * @see SwapsInteractor#getUserSwaps(String, ServiceCallback)
     */
    @Override
    public void getUserSwaps(String userId, final ServiceCallback<List<UiSwap>> callback) {
        mSwapService.getUserSwaps(userId, new ServiceCallback<List<Intercambio>>() {
            @Override
            public void onSuccess(List<Intercambio> response) {
                callback.onSuccess(swapMapper.fromList(response));
            }

            @Override
            public void onError(ServiceFault fault) {
                callback.onError(fault);
            }
        });
    }
}
