package edu.uoc.jsev.b4b.ui.swaps.swapdetail;

import edu.uoc.jsev.b4b.data.uimodel.UiSwap;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BaseContract;

/**
 * Contrato entre la vista y el presentador
 * de la pantalla de detalle de intercambio.
 */
public interface SwapDetailContract extends BaseContract.View {

    /**
     * Vista de la pantalla de detalle de intercambio.
     */
    interface View extends BaseContract.View {
        /**
         * Devuelve los datos del intercambio a mostar en pantalla.
         * @return El intercambio a mostrar.
         */
        UiSwap getSwapData();

        /**
         * Muestra/oculta el botón para seleccionar
         * un libro del otro usuario participante.
         * @param show true para mostrar el botón,
         *             false para ocultarlo.
         */
        void showSearchButton(boolean show);

        /**
         * Elimina todas las opciones del menú de la pantalla.
         */
        void clearOptionsMenu();

        /**
         * Añade al menú de la pantalla la opción
         * de cancelación del intercambio.
         */
        void addCancelMenuItem();

        /**
         * Añade al menú de la pantalla la opción
         * de valoración del intercambio.
         */
        void addRateMenuItem();

        /**
         * Establece la fotografía del libro solicitado por el usuario.
         * @param url La URL de descarga de la fotografía del libro.
         */
        void setBook1Picture(String url);

        /**
         * Establece como fotografía del libro
         * una imagen indicando que el usuario ha rechazado
         * la solicitud de intercambio recibida.
         */
        void setBook1PictureCanceled();

        /**
         * Establece un texto indicando que el usuario
         * ha solicitado un libro al otro participante.
         * @param user El nombre del otro usuario participante.
         *             Este nombre se incluye en el texto mostrado.
         */
        void setBook1Requested(String user);

        /**
         * Establece un texto indicando que el usuario
         * aún no ha solicitado ningún libro al otro participante.
         * @param user El nombre del otro usuario participante.
         *             Este nombre se incluye en el texto mostrado.
         */
        void setBook1Pending(String user);

        /**
         * Establece un texto indicando que el usuario ha cancelado
         * el intercambio solicitado por el otro participante.
         * @param user El nombre del otro usuario participante.
         *             Este nombre se incluye en el texto mostrado.
         */
        void setBook1Canceled(String user);

        /**
         * Establece el título del libro solicitado por el usuario.
         * @param title El título del libro solicitado por el usuario.
         */
        void setBook1Title(String title);

        /**
         * Establece la valoración recibida por el usuario.
         * @param rating La valoración recibida por el usuario.
         */
        void setRating1(String rating);

        /**
         * Establece la fotografía del libro solicitado al usuario
         * por el otro participante.
         * @param url La URL de descarga de la fotografía del libro.
         */
        void setBook2Picture(String url);

        /**
         * Establece como fotografía del libro
         * una imagen indicando que el otro participante
         * ha rechazado la solicitud enviada por el usuario.
         */
        void setBook2PictureCanceled();

        /**
         * Establece un texto indicando que el otro participante
         * ha solicitado un libro al usuario.
         * @param user El nombre del otro usuario participante.
         *             Este nombre se incluye en el texto mostrado.
         */
        void setBook2Requested(String user);

        /**
         * Establece un texto indicando que el otro participante
         * aún no ha solicitado ningún libro al usuario.
         * @param user El nombre del otro usuario participante.
         *             Este nombre se incluye en el texto mostrado.
         */
        void setBook2Pending(String user);

        /**
         * Establece un texto indicando que el otro participante
         * ha cancelado el intercambio solicitado por el usuario.
         * @param user El nombre del otro usuario participante.
         *             Este nombre se incluye en el texto mostrado.
         */
        void setBook2Canceled(String user);

        /**
         * Establece el título del libro solicitado
         * por el otro usuario participante.
         * @param title El título del libro solicitado
         *              por el otro usuario participante.
         */
        void setBook2Title(String title);

        /**
         * Establece la valoración otorgada por el usuario
         * al otro participante.
         * @param rating La valoración otorgada por el usuario
         *               al otro participante.
         */
        void setRating2(String rating);

        /**
         * Muestra el icono de intercambio pendiente.
         */
        void setSwapStatePending();

        /**
         * Muestra el icono de intercambio aceptado.
         */
        void setSwapStateAccepted();

        /**
         * Muestra el icono de intercambio cancelado.
         */
        void setSwapStateCanceled();

        /**
         * Muestra un diálogo de confirmación
         * para cancelar el intercambio.
         */
        void showCancelationDialog();

        /**
         * Muestra un diálogo para valorar el intercambio.
         */
        void showRatingDialog();

        /**
         * Descarta el diálogo de cancelación del intercambio.
         */
        void dismissCancelationDialog();

        /**
         * Descarta el diálogo de valoración del intercambio.
         */
        void dismissRatingDialog();

        /**
         * Muestra un mensaje indicando que el intercambio
         * ha sido cancelado correctamente.
         */
        void showCancelationOk();

        /**
         * Muestra un mensaje indicando que se ha producido
         * un error al cancelar el intercambio.
         */
        void showCancelationError();

        /**
         * Muestra un mensaje indicando que el intercambio
         * ha sido valorado correctamente.
         */
        void showRatingOk();

        /**
         * Muestra un mensaje indicando que se ha producido
         * un error al valorar el intercambio.
         */
        void showRatingError();

        /**
         * Navega hacia el listado de intercambios.
         * @param swap Datos actualizados del intercambio, para
         *             actualizarlos también en la actividad listado.
         *             Si los datos no han sido actualizados,
         *             el parámetro es null y la actividad listado
         *             no realiza ninguna acción.
         */
        void navigateToSwaps(UiSwap swap);

        /**
         * Navega hacia la pantalla de detalle de usuario,
         * para poder seleccionar uno de sus libros.
         * @param user El usuario del que se va a visualizar su detalle.
         * @param swapId El id del intercambio, para adjuntar los datos
         *               del posible libro que se seleccione.
         */
        void navigateToUserDetail(UiUser user, String swapId);
    }

    /**
     * Presentador de la pantalla de detalle de intercambio.
     */
    interface Presenter extends BaseContract.Presenter {
        /**
         * Configura las opciones de menú de la pantalla.
         */
        void setupMenu();

        /**
         * El usuario pulsa sobre la opción de menú
         * para cancelar el intercambio.
         */
        void cancelOptionItemSelected();

        /**
         * El usuario pulsa sobre la opción de menú
         * para valorar el intercambio.
         */
        void rateOptionItemSelected();

        /**
         * El usuario pulsa sobre el botón "Aceptar"
         * del diálogo de confirmación de cancelación
         * del intercambio.
         */
        void cancelationDialogOk();

        /**
         * El usuario pulsa sobre el botón "Cancelar"
         * del diálogo de confirmación de cancelación
         * del intercambio.
         */
        void cancelationDialogCancel();

        /**
         * El usuario pulsa sobre el botón "Aceptar"
         * del diálogo de valoración del intercambio.
         * @param rating La valoración seleccionada
         *               por el usuario.
         */
        void ratingDialogOk(float rating);

        /**
         * El usuario pulsa sobre el botón "Cancelar"
         * del diálogo de valoración del intercambio.
         */
        void ratingDialogCancel();

        /**
         * El usuario pulsa sobre el botón para seleccionar
         * un libro del otro usuario participante en el intercambio.
         */
        void bookSearchButtonPressed();

        /**
         * El usuario ha seleccionado un libro del otro
         * usuario participante en el intercambio.
         * @param bookTitle Título del libro seleccionado.
         * @param bookPicture URL de descarga de la fotografía
         *                    del libro seleccionado.
         */
        void bookRequested(String bookTitle, String bookPicture);
    }
}
