package edu.uoc.jsev.b4b.ui.swaps.swapdetail;

import edu.uoc.jsev.b4b.data.ServiceCallback;
import edu.uoc.jsev.b4b.data.uimodel.UiUser;
import edu.uoc.jsev.b4b.ui.base.BaseSwapsInteractor;

/**
 * Interfaz del interactor de la pantalla de detalle de intercambio.
 */
public interface SwapDetailInteractor extends BaseSwapsInteractor {
    /**
     * Obtiene los datos del usuario participante en el intercambio que no
     * es el usuario actual de la aplicación.
     * @param srcUserId Id del usuario que inició el intercambio.
     * @param dstUserId Id del usuario destinatario del intercambio.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la petición tiene éxito se envía al callback el usuario.
     */
    void findSwapParticipant(String srcUserId, String dstUserId, ServiceCallback<UiUser> callback);

    /**
     * Modifica el estado del intercambio pasado como parámetro,
     * estableciéndolo como cancelado. Si la cancelación es correcta
     * se envía también un correo al usuario que inició el intercambio,
     * informándole de la cancelación.
     * @param swapId Id del intercambio a cancelar.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la modificación es correcta se informa al callback
     *                 por medio de su método onSuccess. Si se produce algún
     *                 error en la modificación se informa al callback a través
     *                 de su método onError.
     */
    void cancelSwap(String swapId, ServiceCallback<Void> callback);

    /**
     * Actualiza el intercambio pasado como parámetro
     * para incluir la valoración que realiza el usuario.
     * El método actualiza también la valoración media del
     * usuario que recibe la valoración y le envía un correo,
     * informándole de que ha recibido una valoración nueva.
     * @param swapId Id del intercambio a valorar.
     * @param userId Id del usuario que recibe la valoración.
     * @param srcRating Valoración recibida por el usuario origen del intercambio.
     * @param dstRating Valoración recibida por el usuario destinatario del intercambio.
     * @param callback Callback al que se devuelve la respuesta del servidor.
     *                 Si la modificación es correcta se informa al callback
     *                 por medio de su método onSuccess. Si se produce algún
     *                 error en la modificación se informa al callback a través
     *                 de su método onError.
     */
    void rateSwap(String swapId, String userId, int srcRating, int dstRating, ServiceCallback<Void> callback);
}
