package edu.uoc.jsev.b4b.util;

/**
 * Clase auxiliar de utilidades para utilizar en mapas.
 */
public class MapUtils {

    // Radio medio de la Tierra en kilómetros.
    private static final int EARTH_RADIUS = 6371;

    /**
     * Obtiene la distancia en kilómetros entre dos puntos terrestres,
     * dadas sus respectivas coordenadas, utilizando la fórmula del haversine.
     *
     * @param lat0 Latitud del primer punto.
     * @param lon0 Longitud del primer punto.
     * @param lat1 Latitud del segundo punto.
     * @param lon1 Longitud del segundo punto.
     * @return Distancia en kilómetros entre los dos puntos.
     */
    public static double distance(double lat0, double lon0, double lat1, double lon1) {
        double dLat = Math.toRadians(lat1 - lat0);
        double dLong = Math.toRadians(lon1 - lon0);

        lat0 = Math.toRadians(lat0);
        lat1 = Math.toRadians(lat1);

        double a = Math.pow(Math.sin(dLat / 2), 2) + Math.cos(lat0) * Math.cos(lat1) * Math.pow(Math.sin(dLong / 2), 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));

        return EARTH_RADIUS * c;
    }

    /**
     * Establece el nivel de zoom de un mapa en función
     * del radio de búsqueda alrededor de un punto central,
     * para que el área sea completamente visible.
     *
     * @param distanceInKms Distancia en kilómetros del radio.
     * @return Nivel de zoom del mapa.
     */
    public static int getZoomLevelFromDistance(int distanceInKms) {
        double scale = 1000 * distanceInKms / 500;
        return (int) (16 - Math.log(scale) / Math.log(2));
    }
}
