package edu.uoc.jsev.b4b;

import org.junit.Test;

import java.io.IOException;
import java.util.Map;

import edu.uoc.jsev.b4b.data.openlibrary.model.OpenLibraryBook;
import edu.uoc.jsev.b4b.data.openlibrary.service.OpenLibraryService;
import retrofit2.Response;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

/**
 * Tests unitarios para la llamada a la API de OpenLibrary.
 * @see OpenLibraryService
 */
public class OpenLibraryTest {

    // Datos para la prueba
    private static final String TEST_ISBN = "9788401337635";
    private static final String TEST_TITLE = "La caida de los Gigantes";
    private static final String TEST_AUTHOR = "Ken Follett";
    private static final String TEST_PUBLISHER = "Random House Mondadori, S.A.";
    private static final String TEST_COVER = "https://covers.openlibrary.org/b/id/8146891-L.jpg";

    @Test
    public void findBookByIsbn_caidaGigantes() throws IOException {
        String queryParam = "ISBN:" + TEST_ISBN;
        Response<Map<String, OpenLibraryBook>> response = OpenLibraryService.getInstance().getBookByIsbn(queryParam).execute();
        assertTrue(response.isSuccessful());

        OpenLibraryBook book = response.body().get(queryParam);

        // Libro
        assertNotNull(book);

        // Título
        assertEquals(TEST_TITLE, book.getTitle());

        // Autor
        assertNotNull(book.getAuthors());
        assertTrue(!book.getAuthors().isEmpty());
        assertEquals(TEST_AUTHOR, book.getAuthors().get(0).getName());

        // Editorial
        assertNotNull(book.getPublishers());
        assertTrue(!book.getPublishers().isEmpty());
        assertEquals(TEST_PUBLISHER, book.getPublishers().get(0).getName());

        // Portada
        assertNotNull(book.getCover());
        assertEquals(TEST_COVER, book.getCover().getLarge());
    }
}
